clear all
close all
clc

%compare asset distribution data from model
%% load data
addpath('F:\Insurance\Revision\Numerics\Clean\Stata')
M = csvread('asset_dist.out');

names = {'id' 'time' 'age' 'emp' 'F wealth' 'earn' 'weight' 'educ' 'EU' 'House Value'};


M(:,11) = M(:,5)+M(:,10); %total net wealth

%% median conditional negative wealth
NW = M;
NW = NW(NW(:,5)<0,:); %keep those with negative wealth


neg_wealth.all = median(NW(:,5));

save('neg_wealth.mat','neg_wealth')

%% employment shares

%urate
A = M(M(:,3)==25,:);
ini.urate = sum(A(A(:,4)==0,7))./(sum(A(:,7)));


%% initial distribution of wealth

%employed
A = M(M(:,3)==25,:);
A(A(:,4)==0,:) = []; %delete unemployed
A = sortrows(A,5);


A_all = A;
A_all(A_all(:,5)<neg_wealth.all,5) = neg_wealth.all;
A_all = sortrows(A_all,5);
ini.emp.all = A_all(:,5);
ini.emp.all(:,2) = 1/length(ini.emp.all);
A_all = sortrows(A_all,10);
ini.emp.all(:,3) = A_all(:,10);
ini.emp.all(:,4) = 1/length(ini.emp.all(:,3));


%unemployed
A = M(M(:,3)==25,:);
A(A(:,4)==1,:) = []; %delete employed

ini.une.all = median(A(:,5));
ini.une.all(:,2) = median(A(:,10));

save('ini.mat','ini');

%% mean and median financial wealth
W = M;

%all
fcn = @(x)median(x);
aux = sortrows(W(:,[5 3]),2);
wealth_data.age.median.all = accumarray_expand_s(aux(:,1),aux(:,2),fcn);
wealth_data.median.all = median(W(:,5));
fcn = @(x)mean(x);
wealth_data.age.mean.all = accumarray_expand_s(aux(:,1),aux(:,2),fcn);
wealth_data.age.mean.all = smoothn(wealth_data.age.mean.all,15);
wealth_data.mean.all = mean(W(:,5));


%% Mean and median wealth housing

W = M;

fcn = @(x)median(x);
aux = sortrows(W(:,[10 3]),2);
wealth_data.h.age.median.all = accumarray_expand_s(aux(:,1),aux(:,2),fcn);
wealth_data.h.median.all = median(W(:,10));
fcn = @(x)mean(x);
wealth_data.h.age.mean.all = accumarray_expand_s(aux(:,1),aux(:,2),fcn);
wealth_data.h.age.mean.all = smoothn(wealth_data.h.age.mean.all,15);
wealth_data.h.mean.all = mean(W(:,10));


%% Households with low wealth

W = M;

%all
fcn = @(x)mean(x);
aux = sortrows(W(:,[5 10 3]),3);
aux(:,4) = aux(:,1)<=3000;
wealth_data.low.all(:,1) = accumarray_expand_s(aux(:,4),aux(:,3),fcn);
wealth_data.low.all(:,1) = smoothn(wealth_data.low.all(:,1),15);
aux(:,4) = aux(:,1)<=10000;
wealth_data.low.all(:,2) = accumarray_expand_s(aux(:,4),aux(:,3),fcn);
wealth_data.low.all(:,2) = smoothn(wealth_data.low.all(:,2),15);
aux(:,1) = aux(:,1); 
aux(:,4) = aux(:,1)<=5000;
wealth_data.low.all(:,3) = accumarray_expand_s(aux(:,4),aux(:,3),fcn);
wealth_data.low.all(:,3) = smoothn(wealth_data.low.all(:,3),15);
aux(:,4) = aux(:,1)<=7000;
wealth_data.low.all(:,4) = accumarray_expand_s(aux(:,4),aux(:,3),fcn);
wealth_data.low.all(:,4) = smoothn(wealth_data.low.all(:,4),15);
aux(:,4) = aux(:,1)<=15000;
wealth_data.low.all(:,5) = accumarray_expand_s(aux(:,4),aux(:,3),fcn);
wealth_data.low.all(:,5) = smoothn(wealth_data.low.all(:,4),15);

%% Wealth inequality

aux = M(:,5);
aux(aux(:,1)<0,1) = 0;
Wealth.gini.fin = ginicoeff(aux(:,1));
aux = M(:,10);
aux(aux(:,1)<0,1) = 0;
Wealth.gini.house = ginicoeff(aux(:,1));
aux = M(:,5)+M(:,10);
aux(aux(:,1)<0,1) = 0;
Wealth.gini.tot = ginicoeff(aux(:,1));

%% Wealth of the employed and unemployed

W = M(logical((M(:,4)==1).*(M(:,3)<=64)),:);
wealth_data.emp.fin.mean = mean(W(:,5));
wealth_data.emp.fin.median = median(W(:,5));
wealth_data.emp.tot.mean = mean(W(:,5)+W(:,10));
wealth_data.emp.tot.median = median(W(:,5)+W(:,10));

W = M(logical((M(:,4)==0).*(M(:,3)<=64)),:);
wealth_data.unemp.fin.mean = mean(W(:,5));
wealth_data.unemp.fin.median = median(W(:,5));
wealth_data.unemp.tot.mean = mean(W(:,5)+W(:,10));
wealth_data.unemp.tot.median = median(W(:,5)+W(:,10));
wealth_data.unemp.fin.shareL = sum(W(:,5)<=3000)./length(W);
wealth_data.unemp.fin.shareL10 = sum(W(:,5)<=10000)./length(W);

%% wealth of the just unemployed

UR = M;

% UR(UR(:,4)>0,:) = []; %no retirement income
UR(UR(:,9)~=-1,:) = []; %keep those just unemployed
UR = UR(UR(:,3)<65,:);

%in case of multiple observations, keep only the last one
UR(:,12) = count_obs(UR(:,1)); %length of spell
UR(:,13) = 1;
fcn = @(x){cumsum(x)}; 
UR(:,14) = accumarray_expand(UR(:,13),UR(:,1),fcn);
UR(:,15) = UR(:,12)==UR(:,14);
UR(UR(:,15)==0,:) = [];
UR(:,11:15) = [];

wealth_data.junemp = zeros(100,2);
for i = 1:length(wealth_data.junemp)
    h = i*100/(length(wealth_data.junemp));
    wealth_data.junemp(i,1) = prctile(UR(:,5),h);
    wealth_data.junemp(i,2) = prctile(UR(:,5)+UR(:,10),h);
end

save('wealth_data.mat','wealth_data');


