clear all
close all
clc
%compute employment transitions (emp=1 employed, emp=0 unemployed)

%% load data
addpath('F:\Insurance\Revision\Numerics\Clean\Stata')

M = csvread('trans.out');

names = {'id' 'time' 'weight' 'emp' 'parti' 'age' 'educ' 'retire' 'di' 'disable'};

M = M(M(:,6)<65,:);
%% unemployment rate over age

X = sortrows(M,6);
fcn = @(x)mean(x);
urate.all = accumarray_expand_s(X(:,4),X(:,6),fcn);
urate.all = 1-urate.all;


%% preparation for tarnsitions

%keep only those with length>1
M(:,11) = count_obs(M(:,1));
M(M(:,11)==1,:) = [];
M(:,11) = [];

%change in employment status (1=found job this end of period, -1=lost job)
fcn = @(x){diff_indi(x,1)};
M(:,11) = accumarray_expand(M(:,4),M(:,1),fcn);
names{11} = 'delta E';


%% E to N 
E = M;

E(E(:,4)~=1,:) = []; %keep employed
E(isnan(E(:,11)),:) = []; %drop last obs of individual
E(:,12) = E(:,11)==0; %employed and did not get unemployed
E(:,13) = E(:,11)==-1; %employed and lost the job (includes OLF)

stay.E = sum(E(:,12))./sum(E(:,12)+E(:,13));
stay.E_health = sum(E(E(:,10)==0,12))./sum(E(E(:,10)==0,12)+E(E(:,10)==0,13));
stay.E_disab = sum(E(E(:,10)==1,12))./sum(E(E(:,10)==1,12)+E(E(:,10)==1,13));

young = E(E(:,6)<50,:);
stay.E_young = sum(young(:,12))./sum(young(:,12)+young(:,13));
stay.E_young_health = sum(young(young(:,10)==0,12))./sum(young(young(:,10)==0,12)+young(young(:,10)==0,13));
stay.E_young_disab = sum(young(young(:,10)==1,12))./sum(young(young(:,10)==1,12)+young(young(:,10)==1,13));

old = E(E(:,6)>=50,:);
stay.E_old = sum(old(:,12))./sum(old(:,12)+old(:,13));
stay.E_old_health = sum(old(old(:,10)==0,12))./sum(old(old(:,10)==0,12)+old(old(:,10)==0,13));
stay.E_old_disab = sum(old(old(:,10)==1,12))./sum(old(old(:,10)==1,12)+old(old(:,10)==1,13));

%% U to E transitions

U = M;
%drop out of labor force
index = U(:,5)==0;
U = U(logical(index),:); 
U(U(:,4)~=0,:) = []; %keep unemployed
U(isnan(U(:,11)),:) = []; %drop last obs of individual
U(:,12) = U(:,11) == 1; %found a job
U(:,13) = U(:,11)==0; %found no job

stay.U = sum(U(:,13))./sum(U(:,12)+U(:,13));
stay.U_health = sum(U(U(:,10)==0,13))./sum(U(U(:,10)==0,12)+U(U(:,10)==0,13));
stay.U_disab = sum(U(U(:,10)==1,13))./sum(U(U(:,10)==1,12)+U(U(:,10)==1,13));

young = U(U(:,6)<50,:);
stay.U_young = sum(young(:,13))./sum(young(:,12)+young(:,13));
stay.U_young_health = sum(young(young(:,10)==0,13))./sum(young(young(:,10)==0,12)+young(young(:,10)==0,13));
stay.U_young_disab = sum(young(young(:,10)==1,13))./sum(young(young(:,10)==1,12)+young(young(:,10)==1,13));

old = U(U(:,6)>=50,:);
stay.U_old = sum(old(:,13))./sum(old(:,12)+old(:,13));
stay.U_old_health = sum(old(old(:,10)==0,13))./sum(old(old(:,10)==0,12)+old(old(:,10)==0,13));
stay.U_old_disab = sum(old(old(:,10)==1,13))./sum(old(old(:,10)==1,12)+old(old(:,10)==1,13));

%% save results

urate_ini = urate.all(1,1);
save('stay.mat','stay');
save('urate_ini.mat','urate_ini');
save('urate.mat','urate');



