clear all
close all
clc
%compute life-cycle properties of means testing
%% load data
addpath('F:\Insurance\Revision\Numerics\Clean\Stata')
M = csvread('w_mt_dist.out');

names = {'id' 'year' 'weight' 'L_income' 'earn_inc' 'means' 'other_mt' 'SSI' 'TANF' 'FOOD' 'WIC' 'ub' 'age' 'educ' 'emp' 'N' 'Pov' 'Income' 'F Wealth', 'MT status', 'DI'};


load('hnp.mat','hnp'); 

M(:,6) = M(:,7)+M(:,8)+M(:,9)+M(:,10)+M(:,11);

life = max(M(:,13))-min(M(:,13))+1;

aux_y=[1 6810/3*1.3; 2 9190/3*1.3; 3 11570/3*1.3; 4 13950/3*1.3];
ybar_y = interp1_l(aux_y(:,1),aux_y(:,2),hnp);
aux_o=[1 6810/3*1.65; 2 9190/3*1.65; 3 11570/3*1.65; 4 13950/3*1.65];
ybar_o = interp1_l(aux_o(:,1),aux_o(:,2),hnp);


%% distribution of wic transfers in working population

WW = M((M(:,11))>0,:);
WW(WW(:,13)>40,:) = []; %delete too old
WW(WW(:,15)~=1,:) = []; %employed
WW(:,17) = WW(:,11); %add wic
WW(:,18) = 0; %delete too high income
for t = min(WW(:,13)):max(WW(:,13))
    WW(WW(:,13)==t,18) = WW(WW(:,13)==t,4)>ybar_y(t-min(WW(:,13))+1);
end


%% distribution of wic transfers in unemployed with benefits population

WUB = M((M(:,11))>0,:);
WUB(WUB(:,15)~=0,:) = []; %unemployed
WUB(WUB(:,13)>40,:) = []; %delete too old
WUB(WUB(:,12)<=1,:) = []; %benefits positive
WUB(:,17) = WUB(:,11); %add wic


%% distribution of wic transfers in unemployed with no benefits population

WU = M((M(:,11))>0,:);
WU(WU(:,13)>40,:) = []; %delete too old
WU(WU(:,15)~=0,:) = []; %unemployed
WU(WU(:,12)>1,:) = []; %no benefits
WU(:,17) = WU(:,11); %add wic


%% mean WIC transfers

mt_wic = mean([WW(:,17); WUB(:,17); WU(:,17)]);
save('mt_wic.mat','mt_wic'); 

%% distribution of other mt transfers in working population

OW = M((M(:,7))>0,:);
OW(OW(:,15)~=1,:) = []; %employed
OW(:,17) = OW(:,7); %add others

mt_other_W = mean(OW(:,17));
 

%% distribution of other mt transfers in unemployed with no benefits population

OU = M((M(:,7))>0,:);
OU(OU(:,15)~=0,:) = []; %unemployed
OU(OU(:,12)>1,:) = []; %no benefits
OU(:,17) = OU(:,7); %add others

mt_other_U = mean(OU(:,17));
  

%% distribution of other mt transfers in unemployed with benefits population

OUB = M((M(:,7))>0,:);
OUB(OUB(:,15)~=0,:) = []; %unemployed
OUB(OUB(:,12)<=1,:) = []; %benefits positive
OUB(:,17) = OUB(:,7); %add others

mt_other_UB = mean(OUB(:,17));


%% distribution of other mt transfers in working population

mt_other = mean([OW(:,17); OUB(:,17); OU(:,17)]);
save('mt_other.mat','mt_other'); 

%% distribution of other mt transfers in retired population

OR = M((M(:,7))>0,:);
OR(OR(:,13)<65,:) = []; %retired
OR(:,17) = OR(:,7); %add others

mt_other_R = mean(OR(:,17));
save('mt_other_R.mat','mt_other_R'); 


%% Share receiving mt over age

O = M;
O = sortrows(O,13);
O(:,6) = sum(O(:,7:11),2); %total MT
O(:,4) = sum(O(:,4),2); %total income

%eligibility
O(:,16) = min(O(:,16),4);
O(:,22) = 0;
O(:,23) = 0;
for i=1:length(O)
    if O(i,13)<66 && O(i,21)==0
        O(i,22) = O(i,4)<aux_y(O(i,16),2);
        O(i,23) = O(i,18)<aux_y(O(i,16),2);
    else
        O(i,22) = O(i,4)<aux_o(O(i,16),2);
        O(i,23) = O(i,18)<aux_o(O(i,16),2);
    end
end
fcn = @(x)mean(x);
data_inc_elig(:,1) = accumarray_expand_s(O(:,22),O(:,13),fcn);
data_inc_elig(:,2) = accumarray_expand_s(O(:,23),O(:,13),fcn);


index3 = O(:,6)>30; %receive MT transfers
index4 = O(:,6)>0; %receive MT transfers
index5 = O(:,20)==1;
data_mt_receive(:,1) = accumarray_expand_s(index3,O(:,13),fcn);
data_mt_receive(:,2) = accumarray_expand_s(index4,O(:,13),fcn);
data_mt_receive(:,3) = accumarray_expand_s(index5,O(:,13),fcn);
for i = 1:3
    data_mt_receive(:,i) = smoothn(data_mt_receive(:,i),15)';
end


save('data_inc_elig.mat','data_inc_elig')
save('data_mt_receive.mat','data_mt_receive')


