clear all
close all
clc
%compute retirement properties of earnings
%% load data
addpath('F:\Insurance\Revision\Numerics\Clean\Stata')
M = csvread('ret_het.out');

names = {'id' 'time' 'age' 'earnings' 'pension_p' 'pension_g' 'ss' 'rail' 'weight' 'educ' 'emp'};


M(M(:,3)<65,:) = []; %retired ages
M(M(:,7)<=50,:) = [];%positive ss
 

%% load grid from model
grid_nc = [100; 387*4; 3500; 1600*4; 2332*4; 13000; 17000];

%compute implies social security income
ss = zeros(length(grid_nc),1);
index = grid_nc<=387*4;
ss(index) = 0.9*grid_nc(index);
index = logical((grid_nc>387*4).*(grid_nc<2333*4));
ss(index) = 0.9*387*4+0.32*(grid_nc(index)-(387*4));
index = (grid_nc>2333*4);
ss(index) = 0.9*387*4+0.32*((2333*4)-(387*4))+0.15*(grid_nc(index)-(2333*4));

M(:,12) = M(:,5)./M(:,7); %pension as share of SS
M(M(:,12)>1,:) = []; %drop those where pension income is main income source

%% differences by grid_nc and age profile
bounds = ss(1:end-1)+diff(ss)/2;
bounds=[0; bounds; 100000000];
aux = M(M(:,3)<95,:);
for i = 1:length(ss)
    pension.nc(i,1) = mean(aux(logical((aux(:,7)>=bounds(i)).*(aux(:,7)<bounds(i+1))),5)); %mean pensions for each gridpoint
end

%get the average age profile of pensions
aux=M(:,[5 7 3]);
aux = sortrows(aux,3);
fcn = @(x)mean(x);
pension.all.age = accumarray_expand_s(aux(:,1),aux(:,3),fcn);
pension.all.age = linspace(pension.all.age(1),pension.all.age(end),length(pension.all.age))';
pension.all.age = pension.all.age./(pension.all.age(round(mean(aux(:,3)))-64));

%adjust average pensions by the age profile
for i = 1:length(grid_nc)
    pension.all.nc.age(:,i)=pension.nc(i,1).*pension.all.age;
end


save('pension.mat','pension')
