#include "fintrf.h"

!----------------------------------------------------------------------------
! Gateway routine
!----------------------------------------------------------------------------

subroutine mexFunction(nlhs,plhs,nrhs,prhs)
    implicit none
    
    ! Arguments in mexFunction
      mwPointer  :: plhs(*), prhs(*)
      integer    :: nlhs, nrhs
    
    ! Function declarations
      mwPointer     mxCreateNumericArray ,mxGetPr, mxGetM, mxGetN
      mwPointer     mxGetDimensions, mxCreateDoubleMatrix
      integer(4)    mxClassIDFromClassName
      mwSize        mxGetNumberOfDimensions
      
      ! Pointers to input/output mxArrays:
      ! Input           
      mwPointer  :: EV,agrid,aprime,na,nap,I,R,gamma,phi,health,hh_scale,hgrid,nh,alpha,hbar,xi &
                    ,abound,s_aprime

       ! Output
      mwPointer  :: V,policy_a,policy_h
      
    ! Array size information
      !integer(4)    complexflag
      integer(4)    :: complexFlag,classid                        ! length of input (output) array
      mwSize        :: ndim
      mwSize        :: dims(2)                    ! dimensions (for mxCreateNumericArray)

      
!--------------------------------------------------------------------------------    
      
      !size of input (and output) vector
      ndim = mxGetNumberOfDimensions(prhs(1))

      ! Retrieve size of input arguments (also size of output argument)
      call mxCopyPtrToPtrArray(mxGetDimensions(prhs(1)), dims, mxGetNumberOfDimensions(prhs(1)))
	  
      classid = mxClassIDFromClassName('double')
      complexFlag = 0  
      
      ! Retrieve input parameter pointers
      EV = mxGetPr(prhs(1))
      agrid = mxGetPr(prhs(2))
      aprime = mxGetPr(prhs(3))
      na = mxGetPr(prhs(4))
      nap = mxGetPr(prhs(5))
      I = mxGetPr(prhs(6))
      R = mxGetPr(prhs(7))
      gamma = mxGetPr(prhs(8))
      phi = mxGetPr(prhs(9))
	  health = mxGetPr(prhs(10))
	  hh_scale = mxGetPr(prhs(11))
      hgrid = mxGetPr(prhs(12))
      nh = mxGetPr(prhs(13))
      alpha = mxGetPr(prhs(14))
      hbar = mxGetPr(prhs(15))
      xi = mxGetPr(prhs(16))
      abound = mxGetPr(prhs(17))
      s_aprime = mxGetPr(prhs(18))
     

     ! Create matrices for return arguments
      plhs(1) = mxCreateNumericArray(ndim, dims, classid, complexflag)
      plhs(2) = mxCreateNumericArray(ndim, dims, classid, complexflag)
	  plhs(3) = mxCreateNumericArray(ndim, dims, classid, complexflag)

	  
      V = mxGetPr(plhs(1))
      policy_a = mxGetPr(plhs(2))
      policy_h = mxGetPr(plhs(3))


      !call computation function
      call  sub_ind_work_test_exog(%val(V),%val(policy_a),%val(policy_h),%val(EV),%val(agrid) &
	                              ,%val(aprime),%val(na),%val(nap),%val(I),%val(R),%val(gamma),%val(phi) &
                                  ,%val(health),%val(hh_scale),%val(hgrid),%val(nh),%val(alpha),%val(hbar) &
                                  ,%val(xi),%val(abound),%val(s_aprime))
        
      return  
      
end subroutine



subroutine sub_ind_work_test_exog(V,policy_a,policy_h,EV,agrid,aprime,na,nap,I,R,gamma,phi,health &
                                  ,hh_scale,hgrid,nh,alpha,hbar,xi,abound,s_aprime)

implicit none

integer(4), intent(in)					      :: na,nap,nh
real(8), intent(in)					          :: R,gamma,phi,health,hh_scale,I,alpha,hbar,xi
real(8), dimension(na), intent(in)			  :: agrid
real(8), dimension(nap), intent(in)			  :: aprime,s_aprime
integer(4), dimension(nap), intent(in)        :: abound
real(8), dimension(nh), intent(in)			  :: hgrid 
real(8), dimension(na,nh), intent(in)	      :: EV
real(8), dimension(na,nh), intent(out)        :: V,policy_a,policy_h


!define further variables
real(8), dimension(nap,nh)              :: c,c2,util,W
real(8), dimension(nap)                 :: v1
integer(4)				                :: i1,i2,i3,i4,i5,i6,i7,index_a,index_h(nap),index_b(2)
real(8)                                 :: util_aux,disutil,gamma_bar,resources,contu
real(8), parameter                      :: minval = 1e-30
       

disutil = exp(phi)
gamma_bar = 1.d0-gamma
call pref(util_aux,minval,hgrid(1),hh_scale,gamma_bar,disutil,alpha,hbar) !utility from infeasible choices

do i1=1,nh !loop over housing STATE
    
    
    do i2=1,na !loop over asset STATE
        
        
        do i3=1,nh
            if (i3==i1) then
                resources = R*agrid(i2)+(I-health)
            else
                resources = R*agrid(i2)+(I-health)+hgrid(i1)-hgrid(i3)-xi
            end if
            
            if ((resources-aprime(1))<0.d0) then !positive consumption not possible
                W(1:nap,i3) = util_aux
            else
                
                i4=0
                do while (i4<nap)
                    i4 = i4+1
                    c(i4,i3) = resources-aprime(i4)
                    if (c(i4,i3)<minval) then !larger choices not possible
                        util(i4:nap,i3) = util_aux
                        W(i4:nap,i3) = util(i4:nap,i3)
                        i4 = nap
                    else
                        call pref(util(i4,i3),c(i4,i3),hgrid(i1) &
                            ,hh_scale,gamma_bar,disutil,alpha,hbar) !utility level for different choices 
                        call lininter2(agrid,EV(:,i3),na,abound(i4),s_aprime(i4),aprime(i4),contu)
                        W(i4,i3) = util(i4,i3)+contu
                    end if
                end do
            end if
        end do
        
        
        !maximum over houses
        do i7 = 1,nap
            call max_val(v1(i7),index_h(i7),W(i7,:),nh)
        end do
        
        !maximum over assets
        call max_val(V(i2,i1),index_a,v1,nap)
        policy_a(i2,i1) = aprime(index_a)
        policy_h(i2,i1) = hgrid(index_h(index_a))
        

    end do
end do



end subroutine



subroutine pref(util,c,h,hh_scale,gamma,disutil,alpha,hbar)

!define in and outputs
real(8), intent(in)    :: c,h,hh_scale,gamma,disutil,alpha,hbar
real(8), intent(out)   :: util

!define further variables
real(8)                :: consu


consu = hh_scale*(((c*disutil)**(alpha))*((hbar+h)**(1-alpha)))

util = (consu**(gamma))/(gamma)


end subroutine



subroutine max_val(y,I,x,na)

!computes maximum (y) and index (I) of vector x, with length na.
!define in and outputs
integer(4), intent(in)	   :: na
real(8), intent(in)    :: x(na)
real(8), intent(out)    :: y
integer(4), intent(out)	   :: I

!define further variables
integer(4)				:: i1

y = x(1)
I = 1

do i1 = 2,na
	if (x(i1)>y) then
		y = x(i1)
		I = i1
	end if
end do

end subroutine
    
    
    
    
subroutine max_val2(y,II1,II2,x,n1,n2)

!computes maximum (y) and index (I1,I2) of matrix x, with dimension (n1,n2).
!define in and outputs
integer(4), intent(in)	                  :: n1,n2
real(8), dimension(n1,n2), intent(in)     :: x
real(8), intent(out)                      :: y
integer, intent(out)	                  :: II1,II2

!define further variables
integer(4)				                  :: i1,i2
integer(4), dimension(n2)                 :: i_aux
real(8), dimension(n2)                    :: y_aux

!find maximum for each column
do i2=1,n2
    y_aux(i2) = x(1,i2)
    i_aux(i2) = 1
    do i1 = 2,n1
	    if (x(i1,i2) > y_aux(i2)) then
		    y_aux(i2) = x(i1,i2)
		    i_aux(i2) = i1
	    end if
    end do
end do

!find maximum across columns
II2 = 1
y = y_aux(1)
do i2 = 2,n2
	if (y_aux(i2) > y) then
		y = y_aux(i2)
        II2 = i2
	end if
end do
II1 = i_aux(II2)


end subroutine
    

subroutine find_pos(vec_in,sca,pos,n)
!finds sca on monotone vector vec_in (number of smaller/equal points)

!define in and outputs
integer, intent(in)	   :: n
real(8), intent(in)    :: vec_in(n),sca
integer, intent(out)   :: pos
integer                :: i1

pos = 0
do i1 = 1,n
	if (sca>=vec_in(i1)) then
		pos = i1
	end if
	if (vec_in(i1)>sca) exit
end do

end subroutine



subroutine lininter2(x,y,n,j,dist,x0,y0)

!Function lininter computes a linear interpolation for point x0
!given the vector of x-values in x and their corresponding funtional values y with length n

implicit none

!define in and outputs
integer, intent(in)						:: n,j
real(8), intent(in)             		:: x(n),y(n)
real(8), intent(in)                     :: x0,dist
real(8), intent(out)                    :: y0


y0 = y(j)+(y(j+1)-y(j))*dist

end subroutine lininter2
    
    
    
subroutine lininter(x,y,n,x0,y0)

!Function lininter computes a linear interpolation for point x0
!given the vector of x-values in x and their corresponding funtional values y with length n

implicit none

!define in and outputs
integer, intent(in)						:: n
real(8), intent(in)             		:: x(n),y(n)
real(8), intent(in)                     :: x0
real(8), intent(out)                    :: y0

!further variables 
integer                                 :: j

!call locate(x,n,x0,j)
call find_pos(x,x0,j,n)

if (j==0) then
    j=1
elseif (j==n) then
    j=n-1
end if

y0 = y(j)+(y(j+1)-y(j))*(x0-x(j))/(x(j+1)-x(j));

end subroutine lininter



subroutine max_int(int_out,int1,int2)

!max of integer

implicit none

!define in and outputs
integer, intent(in)						:: int1,int2
integer, intent(out)                    :: int_out


if (int1>=int2) then
    int_out = int1
else
    int_out = int2
end if

end subroutine max_int


subroutine sub_mexWriteString(String)
  implicit none
  character(*),intent(in):: String
  call mexPrintf(achar(10))
  call mexPrintf(trim(adjustl(String)))
end subroutine sub_mexWriteString