function stay = comp_trans(panel,work,names)

%compute employment transition rates

%drop retired
panel(panel(:,2)==5,:) = [];

%treat disabled as unemployed and sum all unemployed
panel(:,16) = panel(:,2)==1;

%keep only those with length>1
panel(:,17) = count_obs(panel(:,1));
panel(panel(:,17)==1,:) = [];
panel(:,17) = [];

fcn = @(x){diff_indi(x,1)};
panel(:,17) = accumarray_expand(panel(:,16),panel(:,1),fcn); %employment transitions

%% E to N 
E = panel;

E(E(:,2)~=1,:) = []; %keep employed
E(isnan(E(:,17)),:) = []; %drop last obs of individual
E(:,18) = E(:,17)==0; %employed and did not get unemployed
E(:,19) = E(:,17)==-1; %employed and lost the job (includes OLF)

stay.E = sum(E(:,18))./length(E);
stay.E_health = sum(E(E(:,15)==0,18))./sum(E(:,15)==0);
stay.E_disab = sum(E(E(:,15)==1,18))./sum(E(:,15)==1);

young = E(E(:,8)<75,:);
stay.E_young = sum(young(:,18))./length(young);
stay.E_young_health = sum(young(young(:,15)==0,18))./sum(young(:,15)==0);
stay.E_young_disab = sum(young(young(:,15)==1,18))./sum(young(:,15)==1);

old = E(E(:,8)>=75,:);
stay.E_old = sum(old(:,18))./length(old);
stay.E_old_health = sum(old(old(:,15)==0,18))./sum(old(:,15)==0);
stay.E_old_disab = sum(old(old(:,15)==1,18))./sum(old(:,15)==1);


%% U to E transitions

U = panel;
%drop out of labor force and disabled
index = U(:,14)==0;
U = U(logical(index),:); 
index = U(:,2)~=4;
U = U(logical(index),:); 
U(U(:,16)~=0,:) = []; %keep unemployed
U(isnan(U(:,17)),:) = []; %drop last obs of individual
U(:,18) = U(:,17) == 1; %found a job
U(:,19) = U(:,17)==0; %found no job

stay.U = sum(U(:,19))./length(U);
stay.U_health = sum(U(U(:,15)==0,19))./sum(U(:,15)==0);
stay.U_disab = sum(U(U(:,15)==1,19))./sum(U(:,15)==1);

young = U(U(:,8)<75,:);
stay.U_young = sum(young(:,19))./length(young);
stay.U_young_health = sum(young(young(:,15)==0,19))./sum(young(:,15)==0);
stay.U_young_disab = sum(young(young(:,15)==1,19))./sum(young(:,15)==1);

old = U(U(:,8)>=75,:);
stay.U_old = sum(old(:,19))./length(old);
stay.U_old_health = sum(old(old(:,15)==0,19))./sum(old(:,15)==0);
stay.U_old_disab = sum(old(old(:,15)==1,19))./sum(old(:,15)==1);


end