clear all
close all
clc
               
%model with an asset limit of 5000. For describtions of the code, please see
%the file main.m
%% Parameterization
%load in data moments
addpath('F:\Insurance\Revision\Numerics\Clean\Data') %Folder where data is stores
data_di = importdata('di.out'); %shares of those in DI conditional on disability
load('life_determ.mat','life_determ'); %average earnings over life-cycle
load('disable.mat','disable'); %disability rate and transition rates over age 
load('wealth_data.mat','wealth_data'); %wealth moments
load('ini.mat','ini'); %initial unemployment rate and wealth distributions
load('ini_dist.mat','ini_dist'); %initial earnings distribution
load('hnp.mat','hnp'); %number of persosns in a household
hnp = hnp(:,1);
load('earn_u.mat','earn_u'); %earnings and benefits in unemployment
load('urate.mat','urate'); %unemployment rate over the life-cycle
load('stay.mat','stay'); %employment and unemployment transition rates
load('pension.mat','pension'); %pensions
load('life_health.mat','life_health'); %health expenditure over the life-cycle
load('Bell.mat','Bell'); %survival probabilities
load('di_age.mat','di_age'); %share of people in DI
Bell = 1-Bell(1:60,:); %survival probabilities
Bell = 0.63*Bell(:,1)+0.37*Bell(:,2); 


life_determ = life_determ.nodisab;
urate_ini = urate.all(1);
urate_ini_data = urate_ini;
disab_ini = disable.rate(1);

%load parameters from baseline
load('Result_all.mat','Result_all');
urate_ini= Result_all.urate_ini;

%preferences
r = Result_all.r;
R = (1+r);
beta = Result_all.beta; 
theta_b = Result_all.theta_b;
Z = Result_all.Z; 
phi = Result_all.phi; 
alpha = Result_all.alpha;
gamma = Result_all.gamma;     
gamma_b = gamma;
stigma = Result_all.stigma; 
hbar = 2000;
disab_dis = Result_all.disab_dis;

sigma = Result_all.sigma; 
xi = Result_all.xi;


ini_mue = Result_all.ini_mue;
ini_std = Result_all.ini_std;

mu=Result_all.mu;
mu2=Result_all.mu2;
delta1 = min(0,mu);
delta2 = min(0,mu2);

d_cut = 0; 
psi = Result_all.psi; 
psi_d = Result_all.psi_d;


%life
work = 40*3;
retire = 20*3;
disab = 15*3;
old = 15*3;
life = work+retire;


offer_health = zeros(work,1);
offer_disab = zeros(work,1);
offer_health(1:work-old) = (1-stay.U_young_health)*ones(25*3,1); 
offer_health(work-old+1:end) = (1-stay.U_old_health)*ones(old,1); 
offer_disab(1:work-old) = (1-stay.U_young_disab)*ones(25*3,1); 
offer_disab(work-old+1:end) = (1-stay.U_old_disab)*ones(old,1);
destruct_health = Result_all.destruct_health;
destruct_disab = Result_all.destruct_disab;

tran_hd = [disable.getdisable; disable.getdisable(end)];
tran_dh = disable.gethealth;

%survival, health expenditures
survival = zeros(life,1);
hh_size = zeros(life,1);
health_life = zeros(life,1);
for t = 1:length(Bell)
    survival((t-1)*3+1:t*3,1) = Bell(t,1);
    hh_size((t-1)*3+1:t*3,1) = hnp(t,1);
    health_life((t-1)*3+1:t*3,1) = life_health(t,1);
end

survival = survival.^(1/3);
survival(end,1) = 0;

aux = [1 2.631; 2 1.4388; 3 1; 4 0.7899];
hh_scale = interp1_l(aux(:,1),aux(:,2),hh_size);


%income eligible
aux=[1 6810/3*1.3; 2 9190/3*1.3; 3 11570/3*1.3; 4 13950/3*1.3];
ybar_y = interp1_l(aux(:,1),aux(:,2),hh_size);
aux=[1 6810/3*1.65; 2 9190/3*1.65; 3 11570/3*1.65; 4 13950/3*1.65];
ybar_o = interp1_l(aux(:,1),aux(:,2),hh_size);

aux=[1 466*4; 2 689*4; 3 689*4; 4 689*4];
ybar_ss = interp1_l(aux(:,1),aux(:,2),hh_size);


%individual prod grid
N = 17; 
pmin=6.8;
pmax=10.9;

pgrid = linspace(pmin,pmax,N)';
pgrid_d = pgrid-disab_dis;

rho=1;
% Generate transition matrix for non-random walk
pgrid_aux = pgrid-mean(pgrid);
stepp=diff(pgrid_aux);
bounds=[-Inf; pgrid_aux(1:end-1)+stepp/2; Inf];
bounds(1)=bounds(2)-99999999999;
bounds(end)=bounds(end-1)+99999999999;
trans.emp_y = zeros(N,N);
trans.emp_o = zeros(N,N);
trans.unemp_y = zeros(N,N);
trans.unemp_o = zeros(N,N);
for i = 1:N %state
    for j = 1:N %transition to
        trans.emp_y(i,j) = normcdf(bounds(j+1),rho*pgrid_aux(i)+mu,sigma)-normcdf(bounds(j),rho*pgrid_aux(i)+mu,sigma);
        trans.emp_o(i,j) = normcdf(bounds(j+1),rho*pgrid_aux(i)+mu2,sigma)-normcdf(bounds(j),rho*pgrid_aux(i)+mu2,sigma);
        trans.unemp_y(i,j) = normcdf(bounds(j+1),rho*pgrid_aux(i)+delta1,sigma)-normcdf(bounds(j),rho*pgrid_aux(i)+delta1,sigma);
        trans.unemp_o(i,j) = normcdf(bounds(j+1),rho*pgrid_aux(i)+delta2,sigma)-normcdf(bounds(j),rho*pgrid_aux(i)+delta2,sigma);
    end
end

%total earnings grid
nc = 7;
grid_nc = [100; 387*4; 3500; 1600*4; 2332*4; 13000; 17000];
min_nc = min(grid_nc);
max_nc = grid_nc(end);
step_nc = diff(grid_nc);

%housing grid
nh = 11;
hgrid = linspace(0,150000,nh)';
hdiff=diff(hgrid);


%asset grid
amin =  Result_all.amin;
aprime = Result_all.aprime;
agrid = Result_all.agrid;


abar1 = 5000; 
abar2 = 5000; 
[agrid,cut] = get_agrid2(agrid,abar1,abar2);
adiff = diff(agrid);
na = length(agrid);
amax = max(agrid);
[aprime,cut_prime] = get_agrid2(aprime,abar1,abar2);
nap = length(aprime);

abar = zeros(life,1);
abar(1:work,1) = abar1;
abar(work+1:end,1) = abar2;

%get interpolation points for asset choices
[~,abound] = histc(aprime,agrid);
abound(aprime>=agrid(end)) = na-1;
abound(aprime<=agrid(1)) = 1;
s_aprime = (aprime - agrid(abound))./adiff(abound);



%% routine

parpool('local',3)

%unemployment benefits
u_cap = 292*4.3*4; %35.8
b = zeros(N,work);

%unemployment income
earn_u_all = earn_u.all;
earn_u_all(:,2) = earn_u_all(:,2)-earn_u_all(:,1); %earnings in unemployment
w_u = zeros(work,2);
for t = 1:length(earn_u_all)
    w_u((t-1)*3+1:t*3,1) = earn_u_all(t,1);
    w_u((t-1)*3+1:t*3,2) = earn_u_all(t,2);
end
w_u_all = w_u(:,1)+w_u(:,2);


%wages
w = repmat(exp(pgrid),[1 work]);
wd = repmat(exp(pgrid_d),[1 work]);


%social security, disability and pensions
pensions= zeros(nc,retire);
for t = 1:length(pension.all.nc.age)
    pensions(:,(t-1)*3+1:t*3) = repmat(pension.all.nc.age(t,:)',[1 3]);
end

ss = zeros(nc,1);
index = grid_nc<=387*4;
ss(index) = 0.9*grid_nc(index);
index = logical((grid_nc>387*4).*(grid_nc<2333*4));
ss(index) = 0.9*387*4+0.32*(grid_nc(index)-(387*4));
index = (grid_nc>2333*4);
ss(index) = 0.9*387*4+0.32*((2333*4)-(387*4))+0.15*(grid_nc(index)-(2333*4));

w_retire = repmat(ss,[1 retire]);
w_disab = repmat(ss,[1 disab]);

%write into memory income variables from baseline
w_u_original = w_u;
w_original = w;
wd_original = wd;
w_retire_original = w_retire;
w_disab_original = w_disab;


%tolerance levels
tol.all = 0.01;
tol.costs = 0.005;

load('rr_5000.mat','rr_5000') %scaling parameters for benefits
rr=rr_5000;

%loop over the size of the welfare state
crit.all =10000000000;
while crit.all>tol.all
    %% recompute incomes based on generosity of welfare state
    
    %update unemployment benefits
    w_u(:,1)=rr.b*w_u_original(:,1);
    w_u_all = w_u(:,1)+w_u_original(:,2);
    
    
    %update gross disability/retirement income
    aux = rr.ss*w_retire_original;
    wgross_r = aux+pensions;
    wgross_d = rr.d*w_disab_original;
    wgross = w_original;
    wdgross = wd_original;
    
    %net income
    for j = 1:N
        for t = 1:work
            if hh_size(t)<1 %no kids     3
                if wgross(j,t)<=35800/3%less than 35800
                    w(j,t) = 0.85*wgross(j,t);
                elseif wgross(j,t)>35800/3 && wgross(j,t)<=86500/3%less than 86500
                    w(j,t) = wgross(j,t)-1790-0.28*(wgross(j,t)-(35800/3));
                else %more than 86500
                    w(j,t) = wgross(j,t)-6522-0.31*(wgross(j,t)-(86500/3));
                end
                if wdgross(j,t)<=35800/3%less than 35800
                    wd(j,t) = 0.85*wdgross(j,t);
                elseif wdgross(j,t)>35800/3 && wdgross(j,t)<=86500/3%less than 86500
                    wd(j,t) = wdgross(j,t)-1790-0.28*(wdgross(j,t)-(35800/3));
                else %more than 86500
                    wd(j,t) = wdgross(j,t)-6522-0.31*(wdgross(j,t)-(86500/3));
                end
            else
                if wgross(j,t)<=7750/3%less than 7750
                    w(j,t) = 1.035*wgross(j,t);
                elseif wgross(j,t)>7750/3 && wgross(j,t)<=12200/3%less than 12200
                    w(j,t) = 478+0.85*wgross(j,t);
                elseif wgross(j,t)>12200/3 && wgross(j,t)<=23050/3 %less than 23050
                    w(j,t) = 478+0.718*wgross(j,t);
                elseif wgross(j,t)>23050/3 && wgross(j,t)<=35800/3 %less than 35800
                    w(j,t) = 0.85*wgross(j,t);
                elseif wgross(j,t)>35800/3 && wgross(j,t)<=86500/3%less than 86500
                    w(j,t) = wgross(j,t)-1790-0.28*(wgross(j,t)-(35800/3));
                else %more than 86500
                    w(j,t) = wgross(j,t)-6522-0.31*(wgross(j,t)-(86500/3));
                end
                
                if wdgross(j,t)<=7750/3%less than 7750
                    wd(j,t) = 1.035*wdgross(j,t);
                elseif wdgross(j,t)>7750/3 && wdgross(j,t)<=12200/3%less than 12200
                    wd(j,t) = 478+0.85*wdgross(j,t);
                elseif wdgross(j,t)>12200/3 && wdgross(j,t)<=23050/3 %less than 23050
                    wd(j,t) = 478+0.718*wdgross(j,t);
                elseif wdgross(j,t)>23050/3 && wdgross(j,t)<=35800/3 %less than 35800
                    wd(j,t) = 0.85*wdgross(j,t);
                elseif wdgross(j,t)>35800/3 && wdgross(j,t)<=86500/3%less than 86500
                    wd(j,t) = wdgross(j,t)-1790-0.28*(wdgross(j,t)-(35800/3));
                else %more than 86500
                    wd(j,t) = wdgross(j,t)-6522-0.31*(wdgross(j,t)-(86500/3));
                end
            end
        end
    end
    for i = 1:nc
        for t = 1:retire
            if wgross_r(i,t)/2<=35800/3%less than 35800
                w_retire(i,t) = wgross_r(i,t)-0.15*wgross_r(i,t)/2;
            elseif wgross_r(i,t)/2>35800/3 && wgross_r(i,t)/2<=86500/3%less than 86500
                w_retire(i,t) = wgross_r(i,t)-(0.15*35800/6)-0.28*(wgross_r(i,t)/2-(35800/6));
            else %more than 86500
                w_retire(i,t) = wgross_r(i,t)-(0.15*35800/6+0.28*(86500/6-0.15*35800/6))-0.31*(wgross_r(i,t)-(86500/6));
            end
        end
        for t = 1:disab
            if wgross_d(i,t)/2<=35800/3%less than 35800
                w_disab(i,t) = wgross_d(i,t)-0.15*wgross_d(i,t)/2;
            elseif wgross_d(i,t)/2>35800/3 && wgross_d(i,t)/2<=86500/3%less than 86500
                w_disab(i,t) = wgross_d(i,t)-(0.15*35800/6)-0.28*(wgross_d(i,t)/2-(35800/6));
            else %more than 86500
                w_disab(i,t) = wgross_d(i,t)-(0.15*35800/6+0.28*(86500/6-0.15*35800/6))-0.31*(wgross_d(i,t)-(86500/6));
            end
        end
    end
    
    %wage for interpolation
    wL = repmat(reshape(w,[1 N work]),[nc 1 1]);
    grid_ncL = repmat(grid_nc,[1 N]);
    
    %update net wages
    w_retire = w_retire*rr.tau;
    w_disab = w_disab*rr.tau;
    w = w*rr.tau;
    wd = wd*rr.tau;
    
    %egligible to support
    egl_w = zeros(N,work); %number of income states where hh is eglidgible to income support
    for t = 1:work
        for j = 1:N
            egl_w(j,t) = wgross(j,t)<ybar_y(t,1); 
        end
    end
    
    egl_wd = zeros(N,work); %number of income states where hh is eglidgible to income support
    for t = 1:work
        for j = 1:N
            egl_wd(j,t) = wdgross(j,t)<ybar_y(t,1);
        end
    end
    
    egl_b = zeros(work,1); %no benefits
    for t = 1:work
        egl_b(t) = (w_u_all(t,1))<ybar_y(t,1);
    end
    
    egl_d = zeros(nc,disab); %number of income states where hh is eglidgible to income support
    for t = 1:disab
        for i = 1:nc
            egl_d(i,t) = wgross_d(i,t)<ybar_o(work-disab+t,1); 
        end
    end
    
    egl_dss = zeros(nc,disab); %number of income states where hh is eglidgible to ss income support
    for t = 1:disab
        for i = 1:nc
            egl_dss(i,t) = wgross_d(i,t)<ybar_ss(work-disab+t,1); 
        end
    end
    
    egl_r = zeros(nc,retire); %number of income states where hh is eglidgible to income support
    for t = 1:retire
        for i = 1:nc
            egl_r(i,t) = wgross_r(i,t)<ybar_o(work+t,1);
        end
    end
    
    egl_rss = zeros(nc,retire); %number of income states where hh is eglidgible to ss income support
    for t = 1:retire
        for i = 1:nc
            egl_rss(i,t) = wgross_r(i,t)<ybar_ss(work+t,1); 
        end
    end
    
    S = get_transfer(wgross,wdgross,wgross_r,wgross_d,w_u_all,b,N,nc,work,retire,disab,egl_w,egl_wd,egl_r,egl_rss,egl_d,egl_dss,egl_b,hh_size);
    
    S.w = S.w*rr.S;
    S.w_d = S.w_d*rr.S;
    S.d = S.d*rr.S;
    S.r = S.r*rr.S;
    S.u = S.u*rr.S;
    
    %assure that total income is increasing in productivity (have to do it twice)
    for i = 1:2
        aux = w+S.w;
        aux2 = [zeros(1,work); diff(aux,1)];
        index = aux2<0;
        S.w(index) = S.w(index)-aux2(index);
    end
    
    egl_w(S.w>0) = 1; %number of income states where hh is eglidgible to income support
    
    for i = 1:2
        aux = wd+S.w_d;
        aux2 = [zeros(1,work); diff(aux,1)];
        index = aux2<0;
        S.w_d(index) = S.w_d(index)-aux2(index);
    end
    
    egl_wd(S.w_d>0) = 1; %number of income states where hh is eglidgible to income support
    
    %assure that total income is increasing in accumulated earnings (have to do it twice)
    for i = 1:2
        aux = w_disab+S.d;
        aux2 = [zeros(1,disab); diff(aux,1)];
        index = aux2<0;
        S.d(index) = S.d(index)-aux2(index);
    end
    egl_d(S.d>0) = 1;
    
    %assure that total income is increasing in accumulated earnings (have to do it twice)
    for i = 1:2
        aux = w_retire+S.r;
        aux2 = [zeros(1,retire); diff(aux,1)];
        index = aux2<0;
        S.r(index) = S.r(index)-aux2(index);
    end
    egl_r(S.r>0) = 1;
    
      
    %income
    I.w = zeros(N,work); %begining of period income
    I.wd = zeros(N,work); %begining of period income
    I.u = zeros(work,1); %begining of period income
    
    for t = 1:work
        I.u(t,1) = w_u_all(t,1);
        for j = 1:N
            I.w(j,t) = w(j,t);
            I.wd(j,t) = wd(j,t);
        end
    end
    
    I.r = zeros(nc,retire); %begining of period income
    for t = 1:retire
        for i = 1:nc
            I.r(i,t) = w_retire(i,t);
        end
    end
    
    I.d = w_disab;
    
    %%
    %initializations
    V_e = zeros(na,nh,nc,N,work);
    V_e_disab = zeros(na,nh,nc,N,work);
    V_u = zeros(na,nh,nc,N,work);
    V_u_disab = zeros(na,nh,nc,N,work);
    V_uj = zeros(na,nh,nc,N,disab);
    V_uj_disab = zeros(na,nh,nc,N,disab);
    V_d = zeros(na,nh,nc,disab);
    V_r = zeros(na,nh,nc,retire);
    policy_a_e = zeros(na,nh,nc,N,work); %asset policy
    policy_a_u = zeros(na,nh,nc,N,work);
    policy_a_uj = zeros(na,nh,nc,N,disab);
    policy_a_e_disab = zeros(na,nh,nc,N,work);
    policy_a_u_disab = zeros(na,nh,nc,N,work);
    policy_a_uj_disab = zeros(na,nh,nc,N,disab);
    policy_a_d = zeros(na,nh,nc,disab);
    policy_a_r = zeros(na,nh,nc,retire);
    policy_h_e = zeros(na,nh,nc,N,work); %housing policy
    policy_h_u = zeros(na,nh,nc,N,work);
    policy_h_uj = zeros(na,nh,nc,N,disab);
    policy_h_e_disab = zeros(na,nh,nc,N,work);
    policy_h_u_disab = zeros(na,nh,nc,N,work);
    policy_h_uj_disab = zeros(na,nh,nc,N,disab);
    policy_h_d = zeros(na,nh,nc,disab);
    policy_h_r = zeros(na,nh,nc,retire);
    echoice_e = zeros(na,nh,nc,N,work);  %employment policy
    echoice_e2 = zeros(na,nh,nc,N,disab);
    echoice_u = zeros(na,nh,nc,N,work);
    echoice_e_disab = zeros(na,nh,nc,N,work);  %employment policy
    echoice_e2_disab = zeros(na,nh,nc,N,disab);
    echoice_u_disab = zeros(na,nh,nc,N,work);
    d_pol = zeros(na,nh,nc,N,disab);
    d_pol_disab = zeros(na,nh,nc,N,disab);
    policy_s_e = zeros(na,nh,nc,N,work); %asset policy
    policy_s_u = zeros(na,nh,nc,N,work);
    policy_s_uj = zeros(na,nh,nc,N,disab);
    policy_s_d = zeros(na,nh,nc,disab);
    policy_s_r = zeros(na,nh,nc,retire);
    policy_s_e_disab = zeros(na,nh,nc,N,work); %asset policy
    policy_s_u_disab = zeros(na,nh,nc,N,work);
    policy_s_uj_disab = zeros(na,nh,nc,N,disab);
    

    %bequest function
    tw = repmat(agrid,[1 nh])+repmat(hgrid',[na 1]); 
    Z = repmat(Z,[na nh]);
    V_beq = repmat(theta_b*((max(tw+Z,10^(-10))).^(1-gamma_b))./(1-gamma_b),[1 1 nc]);
    
    
    %%
    
    %retirement period
    t = 0;
    tic
    while t<retire
        health_aux = health_life(life-t,1);
        hh_scale_aux = hh_scale(life-t,1);
        
        if t == 0 %last period
            EV = V_beq;
        else
            EV = survival(end-t,1)*V_r(:,:,:,end-t+1)+(1-survival(end-t,1))*V_beq;
        end
        EV = beta*EV;
        
        %auxilary variables for retired
        I_aux = I.r(:,life-work-t);
        S_aux = S.r(:,life-work-t);
        w_egl_aux = egl_r(:,life-work-t);
        abar_aux = abar(life-t,1);
        low_max = cut_prime(2);
        V_aux = zeros(na,nh,nc);
        policy_a_aux = zeros(na,nh,nc);
        policy_h_aux = zeros(na,nh,nc);
        policy_s_aux = zeros(na,nh,nc);
        
        %value function of retired
        parfor j = 1:nc
            if w_egl_aux(j,1)
                
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j),policy_s_aux(:,:,j)] =...
                    asset_test_off_mex(EV(:,:,j),agrid,aprime,int32(low_max),abar_aux,int32(na),int32(nap)...
                    ,I_aux(j,1),0,R,S_aux(j,1),gamma,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,stigma,int32(abound),s_aprime);
                
            else
                
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j)] =...
                    asset_no_off_mex(EV(:,:,j),agrid,aprime,int32(na),int32(nap),I_aux(j,1),R,gamma,0,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,int32(abound),s_aprime);
            end
        end
        
        
        V_r(:,:,:,retire-t) = V_aux;
        policy_a_r(:,:,:,retire-t) = policy_a_aux;
        policy_h_r(:,:,:,retire-t) = policy_h_aux;
        policy_s_r(:,:,:,retire-t) = policy_s_aux;
        
        
        t = t+1;
    end
    
    
    %working life with option of disability
    while t < retire+(disab)
        health_aux = health_life(life-t,1);
        hh_scale_aux = hh_scale(life-t,1);
        if t <retire+old %younger than age 50
            PI_e = trans.emp_o;
            PI_u = trans.unemp_o;
        else
            PI_e = trans.emp_y;
            PI_u = trans.unemp_y;
        end
        
        if t == retire %last period before retirement
            
            %get index and weights to interpolate for employed non-disabled the average life-time earnings
            inter_e_nc = (repmat(w(:,1)',[nc 1])+(work-1)*repmat(grid_nc,[1 N]))./work; %value to be interpolated
            [~,k_e_nc] = histc(inter_e_nc,grid_nc);
            k_e_nc(inter_e_nc<grid_nc(1)) = 1;
            k_e_nc(inter_e_nc>=grid_nc(end)) = nc-1;
            s_e_nc = (inter_e_nc - grid_nc(k_e_nc))./step_nc(k_e_nc);
            
            %expected value functions today employed
            EV_ee = zeros(na,nh,nc,N);
            for i = 1:na
                for j = 1:nh
                    aux2 = squeeze(V_r(i,j,:,1));
                    EV_ee(i,j,:,:) = aux2(k_e_nc) + s_e_nc.*(aux2(k_e_nc+1)-aux2(k_e_nc)); %expected value function employed
                end
            end
            EV_ee = survival(end-t,1)*EV_ee+(1-survival(end-t,1))*repmat(V_beq,[1 1 1 N]);
            EV_euj1 = EV_ee; %expected value function when eligible to DI
            EV_euj2 = EV_ee; %expected value function when non-eligible to DI
            
            %get index and weights to interpolate for employed disabled the average life-time earnings
            inter_e_nc = (repmat(wdgross(:,1)',[nc 1])+(work-1)*repmat(grid_nc,[1 N]))./work; %value to be interpolated
            [~,k_e_nc] = histc(inter_e_nc,grid_nc);
            k_e_nc(inter_e_nc<grid_nc(1)) = 1;
            k_e_nc(inter_e_nc>=grid_nc(end)) = nc-1;
            s_e_nc = (inter_e_nc - grid_nc(k_e_nc))./step_nc(k_e_nc);
            
            %expected value functions today employed disabled
            EV_eed = zeros(na,nh,nc,N);
            for i = 1:na
                for j = 1:nh
                    aux2 = squeeze(V_r(i,j,:,1));
                    EV_eed(i,j,:,:) = aux2(k_e_nc) + s_e_nc.*(aux2(k_e_nc+1)-aux2(k_e_nc)); %expected value function employed
                end
            end
            EV_eed = survival(end-t,1)*EV_eed+(1-survival(end-t,1))*repmat(V_beq,[1 1 1 N]);
            EV_eujd1 = EV_eed;
            EV_eujd2 = EV_eed;
            
            
            %get index and weights to  interpolate for non-employed the average life-time earnings
            inter_u_nc = ((work-1)*repmat(grid_nc,[1 N]))./work;
            [~,k_u_nc] = histc(inter_u_nc,grid_nc);
            k_u_nc(inter_u_nc<grid_nc(1)) = 1;
            k_u_nc(inter_u_nc>=grid_nc(end)) = nc-1;
            s_u_nc = (inter_u_nc - grid_nc(k_u_nc))./step_nc(k_u_nc);
            
            %expected value function unemployed today stazing unemployed
            EV_uu = zeros(na,nh,nc,N);
            for i = 1:na
                for j = 1:nh
                    aux2 = squeeze(V_r(i,j,:,1));
                    EV_uu(i,j,:,:) = aux2(k_u_nc) + s_u_nc.*(aux2(k_u_nc+1)-aux2(k_u_nc)); %expected value function employed
                end
            end
            EV_uu = survival(end-t,1)*EV_uu+(1-survival(end-t,1))*repmat(V_beq,[1 1 1 N]);
            EV_ue = EV_uu; %employment choice irrelavent in last period
            EV_uud = EV_uu; %disability status irrelavent in last period
            EV_ued = EV_uu;
            
            EVD = survival(end-t,1)*V_r(:,:,:,1)+(1-survival(end-t,1))*V_beq; %expected value for those in DI
        else
            %get index and weights to interpolate for employed non-disabled the average life-time earnings
            inter_e_nc = (repmat(w(:,1)',[nc 1])+(work+retire-t-1)*repmat(grid_nc,[1 N]))./(work+retire-t); %value to be interpolated
            [~,k_e_nc] = histc(inter_e_nc,grid_nc);
            k_e_nc(inter_e_nc<grid_nc(1)) = 1;
            k_e_nc(inter_e_nc>=grid_nc(end)) = nc-1;
            s_e_nc = (inter_e_nc - grid_nc(k_e_nc))./step_nc(k_e_nc);
            %stack indexes
            k_e_nc_N = k_e_nc;
            for i = 1:N
                k_e_nc_N(:,i) = k_e_nc(:,i)+(i-1)*nc;
            end
            
            %expected value function employed today
            EV_ee_aux = zeros(na,nh,nc,N); %staying employed
            EV_euj1_aux = zeros(na,nh,nc,N); %becoming unemployed allowed to apply for DI
            EV_euj2_aux = zeros(na,nh,nc,N); %becoming unemployed not allowed to apply for DI
            for i = 1:na
                for j = 1:nh
                    aux2 = squeeze(V_e(i,j,:,:,end+retire-t+1));
                    EV_ee_aux(i,j,:,:) = aux2(k_e_nc_N) + s_e_nc.*(aux2(k_e_nc_N+1)-aux2(k_e_nc_N));
                    aux2 = squeeze(V_uj(i,j,:,:,end+retire-t+1));
                    EV_euj1_aux(i,j,:,:) = aux2(k_e_nc_N) + s_e_nc.*(aux2(k_e_nc_N+1)-aux2(k_e_nc_N));
                    aux2 = squeeze(V_u(i,j,:,:,end+retire-t+1));
                    EV_euj2_aux(i,j,:,:) = aux2(k_e_nc_N) + s_e_nc.*(aux2(k_e_nc_N+1)-aux2(k_e_nc_N));
                end
            end
            
            %get index and weights to interpolate for employed disabled the average life-time earnings
            inter_e_nc = (repmat(wdgross(:,1)',[nc 1])+(work-1)*repmat(grid_nc,[1 N]))./work; %value to be interpolated
            [~,k_e_nc] = histc(inter_e_nc,grid_nc);
            k_e_nc(inter_e_nc<grid_nc(1)) = 1;
            k_e_nc(inter_e_nc>=grid_nc(end)) = nc-1;
            s_e_nc = (inter_e_nc - grid_nc(k_e_nc))./step_nc(k_e_nc);
            %stack indexes
            k_e_nc_N = k_e_nc;
            for i = 1:N
                k_e_nc_N(:,i) = k_e_nc(:,i)+(i-1)*nc;
            end
            
            %expected value function employed disabled today
            EV_eed_aux = zeros(na,nh,nc,N); %staying employed but become diabled
            EV_eujd1_aux = zeros(na,nh,nc,N); %becoming unemployed allowed to apply for DI
            EV_eujd2_aux = zeros(na,nh,nc,N); %becoming unemployed not allowed to apply for DI
            for i = 1:na
                for j = 1:nh
                    aux2 = squeeze(V_e_disab(i,j,:,:,end+retire-t+1));
                    EV_eed_aux(i,j,:,:) = aux2(k_e_nc_N) + s_e_nc.*(aux2(k_e_nc_N+1)-aux2(k_e_nc_N));
                    aux2 = squeeze(V_uj_disab(i,j,:,:,end+retire-t+1));
                    EV_eujd1_aux(i,j,:,:) = aux2(k_e_nc_N) + s_e_nc.*(aux2(k_e_nc_N+1)-aux2(k_e_nc_N));
                    aux2 = squeeze(V_u_disab(i,j,:,:,end+retire-t+1));
                    EV_eujd2_aux(i,j,:,:) = aux2(k_e_nc_N) + s_e_nc.*(aux2(k_e_nc_N+1)-aux2(k_e_nc_N));
                end
            end
            aux_beq = (1-survival(end-t,1))*repmat(V_beq,[1 1 1 N]);
            EV_ee = survival(end-t,1)*((1-tran_hd(end+retire-t+1))*EV_ee_aux+tran_hd(end+retire-t+1)*EV_eed_aux)+aux_beq;
            EV_euj1 = survival(end-t,1)*((1-tran_hd(end+retire-t+1))*EV_euj1_aux+tran_hd(end+retire-t+1)*EV_eujd1_aux)+aux_beq;
            EV_euj2 = survival(end-t,1)*((1-tran_hd(end+retire-t+1))*EV_euj2_aux+tran_hd(end+retire-t+1)*EV_eujd2_aux)+aux_beq;
            EV_eed = survival(end-t,1)*((1-tran_dh(end+retire-t+1))*EV_eed_aux+tran_dh(end+retire-t+1)*EV_ee_aux)+aux_beq;
            EV_eujd1 = survival(end-t,1)*((1-tran_dh(end+retire-t+1))*EV_eujd1_aux+tran_dh(end+retire-t+1)*EV_euj1_aux)+aux_beq;
            EV_eujd2 = survival(end-t,1)*((1-tran_dh(end+retire-t+1))*EV_eujd2_aux+tran_dh(end+retire-t+1)*EV_euj2_aux)+aux_beq;
            
            
            %get index and weights to interpolate for non-employed the average life-time earnings
            inter_u_nc = ((work+retire-t-1)*repmat(grid_nc,[1 N]))./(work+retire-t);
            [~,k_u_nc] = histc(inter_u_nc,grid_nc);
            k_u_nc(inter_u_nc<grid_nc(1)) = 1;
            k_u_nc(inter_u_nc>=grid_nc(end)) = nc-1;
            s_u_nc = (inter_u_nc - grid_nc(k_u_nc))./step_nc(k_u_nc);
            %stack indexes
            k_u_nc_N = k_u_nc;
            for i = 1:N
                k_u_nc_N(:,i) = k_u_nc(:,i)+(i-1)*nc;%add nc states to move to N state
            end
            
            %expected value function non-employed today
            EV_uu_aux = zeros(na,nh,nc,N); %staying non-employed and being non-disabled
            EV_ue_aux = zeros(na,nh,nc,N); %becoming employed and being non-disabled
            EV_uud_aux = zeros(na,nh,nc,N); %staying non-employed and being disabled
            EV_ued_aux = zeros(na,nh,nc,N); %becoming employed and being disabled
            for i = 1:na
                for j = 1:nh
                    aux2 = squeeze(V_u(i,j,:,:,end+retire-t+1));
                    EV_uu_aux(i,j,:,:) = aux2(k_u_nc_N) + s_u_nc.*(aux2(k_u_nc_N+1)-aux2(k_u_nc_N));
                    aux2 = squeeze(V_e(i,j,:,:,end+retire-t+1));
                    EV_ue_aux(i,j,:,:) = aux2(k_u_nc_N) + s_u_nc.*(aux2(k_u_nc_N+1)-aux2(k_u_nc_N));
                    aux2 = squeeze(V_u_disab(i,j,:,:,end+retire-t+1));
                    EV_uud_aux(i,j,:,:) = aux2(k_u_nc_N) + s_u_nc.*(aux2(k_u_nc_N+1)-aux2(k_u_nc_N));
                    aux2 = squeeze(V_e_disab(i,j,:,:,end+retire-t+1));
                    EV_ued_aux(i,j,:,:) = aux2(k_u_nc_N) + s_u_nc.*(aux2(k_u_nc_N+1)-aux2(k_u_nc_N));
                end
            end
            EV_uu = survival(end-t,1)*((1-tran_hd(end+retire-t+1))*EV_uu_aux+tran_hd(end+retire-t+1)*EV_uud_aux)+aux_beq;
            EV_ue = survival(end-t,1)*((1-tran_hd(end+retire-t+1))*EV_ue_aux+tran_hd(end+retire-t+1)*EV_ued_aux)+aux_beq;
            EV_uud = survival(end-t,1)*((1-tran_dh(end+retire-t+1))*EV_uud_aux+tran_dh(end+retire-t+1)*EV_uu_aux)+aux_beq;
            EV_ued = survival(end-t,1)*((1-tran_dh(end+retire-t+1))*EV_ued_aux+tran_dh(end+retire-t+1)*EV_ue_aux)+aux_beq;
            
            EVD = survival(end-t,1)*V_d(:,:,:,end+retire-t+1)+(1-survival(end-t,1))*V_beq;
        end
        
        %employment choice non-employed
        echoice_u(:,:,:,:,end+retire-t) = EV_ue>=EV_uu; %non-disabled
        echoice_u_disab(:,:,:,:,end+retire-t) = EV_ued>=EV_uud; %disabled
        upper_u = max(EV_ue,EV_uu);
        upper_ud = max(EV_ued,EV_uud);
        
        %productivity transition
        U_n = expected(EV_uu,PI_u,N,na,nh,nc); %value of no offer
        U_f = expected(upper_u,PI_u,N,na,nh,nc); %value of offer
        U_nd = expected(EV_uud,PI_u,N,na,nh,nc); %value of no offer disabled
        U_fd = expected(upper_ud,PI_u,N,na,nh,nc); %value of offer disabled
        
        %employment choice employed
        upper1 = max(EV_ee,EV_euj1); %option of DI, non-disabled
        upper2 = max(EV_ee,EV_euj2);  %no option of DI, non-disabled
        upperd1 = max(EV_eed,EV_eujd1); %option of DI, disabled
        upperd2 = max(EV_eed,EV_eujd2); %no option of DI, disabled
        
        echoice_e(:,:,:,:,end+retire-t) = EV_ee>=EV_euj2; %employment choice no option DI, non-disabled
        echoice_e2(:,:,:,:,end+retire-t) = EV_ee>=EV_euj1; %employment choice option DI, non-disabled
        echoice_e_disab(:,:,:,:,end+retire-t) = EV_eed>=EV_eujd2; %employment choice no option DI, disabled
        echoice_e2_disab(:,:,:,:,end+retire-t) = EV_eed>=EV_eujd1; %employment choice option DI, disabled
        
        %productivity transition unemployed
        E_e = zeros(na,nh,nc,N); %value of no destruction non-disabled
        E_u = zeros(na,nh,nc,N); %value of destruction non-disabeld
        E_ed = zeros(na,nh,nc,N); %value of no destruction disabled
        E_ud = zeros(na,nh,nc,N); %value of destruction disabled
        for i = 1:N %state
            for j = 1:N %transition
                if pgrid(i)-pgrid(j)<d_cut %eligible for DI
                    E_u(:,:,:,i) = E_u(:,:,:,i)+EV_euj1(:,:,:,j)*PI_e(i,j);
                    E_e(:,:,:,i) = E_e(:,:,:,i)+upper1(:,:,:,j)*PI_e(i,j);
                    E_ud(:,:,:,i) = E_ud(:,:,:,i)+EV_eujd1(:,:,:,j)*PI_e(i,j);
                    E_ed(:,:,:,i) = E_ed(:,:,:,i)+upperd1(:,:,:,j)*PI_e(i,j);
                else %not eligible for DI
                    E_u(:,:,:,i) = E_u(:,:,:,i)+EV_euj2(:,:,:,j)*PI_e(i,j);
                    E_e(:,:,:,i) = E_e(:,:,:,i)+upper2(:,:,:,j)*PI_e(i,j);
                    E_ud(:,:,:,i) = E_ud(:,:,:,i)+EV_eujd2(:,:,:,j)*PI_e(i,j);
                    E_ed(:,:,:,i) = E_ed(:,:,:,i)+upperd2(:,:,:,j)*PI_e(i,j);
                end
            end
        end
        
        %expected value function unemployed
        EVU = beta*(offer_health(life-t)*U_f+(1-offer_health(life-t))*U_n); %non-disabled
        EVUd = beta*(offer_disab(life-t)*U_fd+(1-offer_disab(life-t))*U_nd); %disabled
        
        abar_aux = abar(life-t,1);
        
        %auxilary variables for long term unemployment
        low_max = cut_prime(1);
        S_aux = S.u(life-t);
        egl_b_aux = egl_b(life-t);
        I_aux = I.u(life-t);
        EVU_aux = reshape(EVU,[na nh nc*N]);
        V_aux = zeros(na,nh,nc*N);
        policy_a_aux = zeros(na,nh,nc*N);
        policy_h_aux = zeros(na,nh,nc*N);
        policy_s_aux = zeros(na,nh,nc*N);
        
        %value function longterm non-employed, non-disabled
        parfor j = 1:nc*N
            if egl_b_aux
                
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j),policy_s_aux(:,:,j)] =...
                    asset_test_off_mex(EVU_aux(:,:,j),agrid,aprime,int32(low_max),abar_aux,int32(na),int32(nap)...
                    ,I_aux,0,R,S_aux,gamma,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,stigma,int32(abound),s_aprime);
                
            else
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j)] =...
                    asset_no_off_mex(EVU_aux(:,:,j),agrid,aprime,int32(na),int32(nap),I_aux,R,gamma,0,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,int32(abound),s_aprime);
                
            end
            
        end
        V_u(:,:,:,:,life-t) = reshape(V_aux,[na nh nc N]);
        policy_a_u(:,:,:,:,life-t) = reshape(policy_a_aux,[na nh nc N]);
        policy_h_u(:,:,:,:,life-t) = reshape(policy_h_aux,[na nh nc N]);
        policy_s_u(:,:,:,:,life-t) = reshape(policy_s_aux,[na nh nc N]);
        
        
        %continuation value just unemployed, non-disabled
        aux = (psi*repmat(EVD,[1 1 1 N])+(1-psi)*U_n);
        EVUJ = (offer_health(life-t)*U_f+(1-offer_health(life-t))*U_n); %continuation value of search
        d_pol(:,:,:,:,end+retire-t) = aux>=EVUJ;
        EVUJ = beta*(max(aux,EVUJ));
        
        %value function long term non-employed disabled
        EVUd_aux = reshape(EVUd,[na nh nc*N]);
        V_aux = zeros(na,nh,nc*N);
        policy_a_aux = zeros(na,nh,nc*N);
        policy_h_aux = zeros(na,nh,nc*N);
        policy_s_aux = zeros(na,nh,nc*N);
        parfor j = 1:nc*N
            if egl_b_aux
                
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j),policy_s_aux(:,:,j)] =...
                    asset_test_off_mex(EVUd_aux(:,:,j),agrid,aprime,int32(low_max),abar_aux,int32(na),int32(nap)...
                    ,I_aux,0,R,S_aux,gamma,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,stigma,int32(abound),s_aprime);
                
            else
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j)] =...
                    asset_no_off_mex(EVUd_aux(:,:,j),agrid,aprime,int32(na),int32(nap),I_aux,R,gamma,0,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,int32(abound),s_aprime);
                
            end
            
        end
        V_u_disab(:,:,:,:,life-t) = reshape(V_aux,[na nh nc N]);
        policy_a_u_disab(:,:,:,:,life-t) = reshape(policy_a_aux,[na nh nc N]);
        policy_h_u_disab(:,:,:,:,life-t) = reshape(policy_h_aux,[na nh nc N]);
        policy_s_u_disab(:,:,:,:,life-t) = reshape(policy_s_aux,[na nh nc N]);
        
        %continuation value just unemployed disabled
        aux = (psi_d*repmat(EVD,[1 1 1 N])+(1-psi_d)*U_nd);
        EVUJd = (offer_disab(life-t)*U_fd+(1-offer_disab(life-t))*U_nd); %continuation value of search
        d_pol_disab(:,:,:,:,end+retire-t) = aux>=EVUJd;
        EVUJd = beta*(max(aux,EVUJd));
        
        %auxilary variables for just unemployment non-disabled
        egl_b_aux = egl_b(life-t);
        S_aux = S.u(life-t);
        I_aux = I.u(life-t);
        low_max = cut_prime(1);
        EVUJ_aux = reshape(EVUJ,[na nh nc*N]);
        V_aux = zeros(na,nh,nc*N);
        policy_a_aux = zeros(na,nh,nc*N);
        policy_h_aux = zeros(na,nh,nc*N);
        policy_s_aux = zeros(na,nh,nc*N);
        
        %value function just-unemployed non-disabled
        parfor j = 1:nc*N
            if egl_b_aux
                
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j),policy_s_aux(:,:,j)] =...
                    asset_test_off_mex(EVUJ_aux(:,:,j),agrid,aprime,int32(low_max),abar_aux,int32(na),int32(nap)...
                    ,I_aux,0,R,S_aux,gamma,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,stigma,int32(abound),s_aprime);
                
                
            else
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j)] =...
                    asset_no_off_mex(EVUJ_aux(:,:,j),agrid,aprime,int32(na),int32(nap),I_aux,R,gamma,0,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,int32(abound),s_aprime);
                
            end
        end
        V_uj(:,:,:,:,retire+disab-t) = reshape(V_aux,[na nh nc N]);
        policy_a_uj(:,:,:,:,retire+disab-t) = reshape(policy_a_aux,[na nh nc N]);
        policy_h_uj(:,:,:,:,retire+disab-t) = reshape(policy_h_aux,[na nh nc N]);
        policy_s_uj(:,:,:,:,retire+disab-t) = reshape(policy_s_aux,[na nh nc N]);
        
        
        
        %value function just unemployed disabled
        EVUJ_aux = reshape(EVUJd,[na nh nc*N]);
        V_aux = zeros(na,nh,nc*N);
        policy_a_aux = zeros(na,nh,nc*N);
        policy_h_aux = zeros(na,nh,nc*N);
        policy_s_aux = zeros(na,nh,nc*N);
        parfor j = 1:nc*N
            if egl_b_aux
                
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j),policy_s_aux(:,:,j)] =...
                    asset_test_off_mex(EVUJ_aux(:,:,j),agrid,aprime,int32(low_max),abar_aux,int32(na),int32(nap)...
                    ,I_aux,0,R,S_aux,gamma,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,stigma,int32(abound),s_aprime);
                
                
                
            else
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j)] =...
                    asset_no_off_mex(EVUJ_aux(:,:,j),agrid,aprime,int32(na),int32(nap),I_aux,R,gamma,0,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,int32(abound),s_aprime);
                
                
            end
        end
        V_uj_disab(:,:,:,:,retire+disab-t) = reshape(V_aux,[na nh nc N]);
        policy_a_uj_disab(:,:,:,:,retire+disab-t) = reshape(policy_a_aux,[na nh nc N]);
        policy_h_uj_disab(:,:,:,:,retire+disab-t) = reshape(policy_h_aux,[na nh nc N]);
        policy_s_uj_disab(:,:,:,:,retire+disab-t) = reshape(policy_s_aux,[na nh nc N]);
        
        %continuation value employed
        EVE = beta*((1-destruct_health(life-t,1))*E_e+destruct_health(life-t,1)*E_u); %non-disabled
        EVEd = beta*((1-destruct_disab(life-t,1))*E_ed+destruct_disab(life-t,1)*E_ud); %disabled
        
        %create auxilary variables for employment
        egl_w_aux = repmat(reshape(egl_w(:,life-t),[1 N]),[nc 1]);
        egl_w_aux = egl_w_aux(:);
        I_aux = repmat(reshape(I.w(:,life-t),[1 N]),[nc 1]);
        I_aux = I_aux(:);
        S_aux = repmat(reshape(S.w(:,life-t),[1 N]),[nc 1]);
        S_aux = S_aux(:);
        low_max = cut_prime(1);
        EVE_aux = reshape(EVE,[na nh nc*N]);
        V_aux = zeros(na,nh,nc*N);
        policy_a_aux = zeros(na,nh,nc*N);
        policy_h_aux = zeros(na,nh,nc*N);
        policy_s_aux = zeros(na,nh,nc*N);
        
        %value function employed non-disabled
        parfor j = 1:nc*N
            if egl_w_aux(j,1)
                
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j),policy_s_aux(:,:,j)] =...
                    asset_test_off_mex(EVE_aux(:,:,j),agrid,aprime,int32(low_max),abar_aux,int32(na),int32(nap)...
                    ,I_aux(j,1),phi,R,S_aux(j,1),gamma,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,stigma,int32(abound),s_aprime);
                
                
            else
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j)] =...
                    asset_no_off_mex(EVE_aux(:,:,j),agrid,aprime,int32(na),int32(nap),I_aux(j,1),R,gamma,phi,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,int32(abound),s_aprime);
                
                
            end
        end
        V_e(:,:,:,:,life-t) = reshape(V_aux,[na nh nc N]);
        policy_a_e(:,:,:,:,life-t) = reshape(policy_a_aux,[na nh nc N]);
        policy_h_e(:,:,:,:,life-t) = reshape(policy_h_aux,[na nh nc N]);
        policy_s_e(:,:,:,:,life-t) = reshape(policy_s_aux,[na nh nc N]);
        
        
        %create auxilary variables for employment disabled
        egl_w_aux = repmat(reshape(egl_wd(:,life-t),[1 N]),[nc 1]);
        egl_w_aux = egl_w_aux(:);
        I_aux = repmat(reshape(I.wd(:,life-t),[1 N]),[nc 1]);
        I_aux = I_aux(:);
        S_aux = repmat(reshape(S.w_d(:,life-t),[1 N]),[nc 1]);
        S_aux = S_aux(:);
        low_max = cut_prime(1);
        EVE_aux = reshape(EVEd,[na nh nc*N]);
        V_aux = zeros(na,nh,nc*N);
        policy_a_aux = zeros(na,nh,nc*N);
        policy_h_aux = zeros(na,nh,nc*N);
        policy_s_aux = zeros(na,nh,nc*N);
        
        %value function employed, disabled
        parfor j = 1:nc*N
            if egl_w_aux(j,1)
                
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j),policy_s_aux(:,:,j)] =...
                    asset_test_off_mex(EVE_aux(:,:,j),agrid,aprime,int32(low_max),abar_aux,int32(na),int32(nap)...
                    ,I_aux(j,1),phi,R,S_aux(j,1),gamma,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,stigma,int32(abound),s_aprime);
                
            else
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j)] =...
                    asset_no_off_mex(EVE_aux(:,:,j),agrid,aprime,int32(na),int32(nap),I_aux(j,1),R,gamma,phi,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,int32(abound),s_aprime);
                
            end
        end
        V_e_disab(:,:,:,:,life-t) = reshape(V_aux,[na nh nc N]);
        policy_a_e_disab(:,:,:,:,life-t) = reshape(policy_a_aux,[na nh nc N]);
        policy_h_e_disab(:,:,:,:,life-t) = reshape(policy_h_aux,[na nh nc N]);
        policy_s_e_disab(:,:,:,:,life-t) = reshape(policy_s_aux,[na nh nc N]);
        
        %continuation value disabled
        EVD = beta*EVD;
        
        %auxilary for disabled
        low_max = cut_prime(1); %low_max_d(:,retire+disab-t);
        S_aux = S.d(:,retire+disab-t);
        d_egl_aux = egl_d(:,retire+disab-t);
        I_aux = I.d(:,retire+disab-t);
        EVD_aux = EVD;
        V_aux = zeros(na,nh,nc);
        policy_a_aux = zeros(na,nh,nc);
        policy_h_aux = zeros(na,nh,nc);
        policy_s_aux = zeros(na,nh,nc);
        
        parfor j = 1:nc
            if d_egl_aux(j,1)
                
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j),policy_s_aux(:,:,j)] =...
                    asset_test_off_mex(EVD_aux(:,:,j),agrid,aprime,int32(low_max),abar_aux,int32(na),int32(nap)...
                    ,I_aux(j,1),0,R,S_aux(j,1),gamma,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,stigma,int32(abound),s_aprime);
                
            else
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j)] =...
                    asset_no_off_mex(EVD_aux(:,:,j),agrid,aprime,int32(na),int32(nap),I_aux(j,1),R,gamma,0,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,int32(abound),s_aprime);
                
            end
            
        end
        V_d(:,:,:,retire+disab-t) = V_aux;
        policy_a_d(:,:,:,retire+disab-t) = policy_a_aux;
        policy_h_d(:,:,:,retire+disab-t) = policy_h_aux;
        policy_s_d(:,:,:,retire+disab-t) = policy_s_aux;
        
        
        t= t+1;
    end
    
    
    
    %working life without option for DI
    while t < life
        
        health_aux = health_life(life-t,1);
        hh_scale_aux = hh_scale(life-t,1);
        abar_aux = abar(life-t,1);
        
        PI_e = trans.emp_y;
        PI_u = trans.unemp_y;
        
        %get index and weights to interpolate employed, non-disabled earning on social security grid
        inter_e_nc = (repmat(wgross(:,1)',[nc 1])+(work+retire-t-1)*repmat(grid_nc,[1 N]))./(work+retire-t); %value to be interpolated
        [~,k_e_nc] = histc(inter_e_nc,grid_nc);
        k_e_nc(inter_e_nc<grid_nc(1)) = 1;
        k_e_nc(inter_e_nc>=grid_nc(end)) = nc-1;
        s_e_nc = (inter_e_nc - grid_nc(k_e_nc))./step_nc(k_e_nc);
        %stack indexes
        k_e_nc_N = k_e_nc;
        for i = 1:N
            k_e_nc_N(:,i) = k_e_nc(:,i)+(i-1)*nc;%add nc states to move to N state
        end
        
        %expected value function employed today
        EV_ee_aux = zeros(na,nh,nc,N); %staying employed
        EV_euj_aux = zeros(na,nh,nc,N); %beconing non-employed
        for i = 1:na
            for j = 1:nh
                aux2 = squeeze(V_e(i,j,:,:,end+retire-t+1));
                EV_ee_aux(i,j,:,:) = aux2(k_e_nc_N) + s_e_nc.*(aux2(k_e_nc_N+1)-aux2(k_e_nc_N));
                aux2 = squeeze(V_u(i,j,:,:,end+retire-t+1));
                EV_euj_aux(i,j,:,:) = aux2(k_e_nc_N) + s_e_nc.*(aux2(k_e_nc_N+1)-aux2(k_e_nc_N));
            end
        end
        
        %%get index and weights to interpolate employed, disabled earning on social security grid
        inter_e_nc = (repmat(wdgross(:,1)',[nc 1])+(work-1)*repmat(grid_nc,[1 N]))./work; %value to be interpolated
        [~,k_e_nc] = histc(inter_e_nc,grid_nc);
        k_e_nc(inter_e_nc<grid_nc(1)) = 1;
        k_e_nc(inter_e_nc>=grid_nc(end)) = nc-1;
        s_e_nc = (inter_e_nc - grid_nc(k_e_nc))./step_nc(k_e_nc);
        %stack indexes
        k_e_nc_N = k_e_nc;
        for i = 1:N
            k_e_nc_N(:,i) = k_e_nc(:,i)+(i-1)*nc;%add nc states to move to N state
        end
        
        %expected value function employed and disabled today
        EV_eed_aux = zeros(na,nh,nc,N); %staying employed
        EV_eujd_aux = zeros(na,nh,nc,N); %becoming non-employed
        for i = 1:na
            for j = 1:nh
                aux2 = squeeze(V_e_disab(i,j,:,:,end+retire-t+1));
                EV_eed_aux(i,j,:,:) = aux2(k_e_nc_N) + s_e_nc.*(aux2(k_e_nc_N+1)-aux2(k_e_nc_N));
                aux2 = squeeze(V_u_disab(i,j,:,:,end+retire-t+1));
                EV_eujd_aux(i,j,:,:) = aux2(k_e_nc_N) + s_e_nc.*(aux2(k_e_nc_N+1)-aux2(k_e_nc_N));
            end
        end
        aux_beq = (1-survival(end-t,1))*repmat(V_beq,[1 1 1 N]);
        EV_ee = survival(end-t,1)*((1-tran_hd(end+retire-t+1))*EV_ee_aux+tran_hd(end+retire-t+1)*EV_eed_aux)+aux_beq;
        EV_euj = survival(end-t,1)*((1-tran_hd(end+retire-t+1))*EV_euj_aux+tran_hd(end+retire-t+1)*EV_eujd_aux)+aux_beq;
        EV_eed = survival(end-t,1)*((1-tran_dh(end+retire-t+1))*EV_eed_aux+tran_dh(end+retire-t+1)*EV_ee_aux)+aux_beq;
        EV_eujd = survival(end-t,1)*((1-tran_dh(end+retire-t+1))*EV_eujd_aux+tran_dh(end+retire-t+1)*EV_euj_aux)+aux_beq;
        
        
        %%get index and weights to interpolate non-employed, non-disabled earning on social security grid
        inter_u_nc = ((work+retire-t-1)*repmat(grid_nc,[1 N]))./(work+retire-t);
        [~,k_u_nc] = histc(inter_u_nc,grid_nc);
        k_u_nc(inter_u_nc<grid_nc(1)) = 1;
        k_u_nc(inter_u_nc>=grid_nc(end)) = nc-1;
        s_u_nc = (inter_u_nc - grid_nc(k_u_nc))./step_nc(k_u_nc);
        %stack index
        k_u_nc_N = k_u_nc;
        for i = 1:N
            k_u_nc_N(:,i) = k_u_nc(:,i)+(i-1)*nc;%add nc states to move to N state
        end
        
        %expected value of non-employed today
        EV_uu_aux = zeros(na,nh,nc,N); %staying non-employed, non-disabled
        EV_ue_aux = zeros(na,nh,nc,N);  %not staying non-employed, non-disabled
        EV_uud_aux = zeros(na,nh,nc,N);  %staying non-employed, disabled
        EV_ued_aux = zeros(na,nh,nc,N);  %not staying non-employed, disabled
        for i = 1:na
            for j = 1:nh
                aux2 = squeeze(V_u(i,j,:,:,end+retire-t+1));
                EV_uu_aux(i,j,:,:) = aux2(k_u_nc_N) + s_u_nc.*(aux2(k_u_nc_N+1)-aux2(k_u_nc_N));
                aux2 = squeeze(V_e(i,j,:,:,end+retire-t+1));
                EV_ue_aux(i,j,:,:) = aux2(k_u_nc_N) + s_u_nc.*(aux2(k_u_nc_N+1)-aux2(k_u_nc_N));
                aux2 = squeeze(V_u_disab(i,j,:,:,end+retire-t+1));
                EV_uud_aux(i,j,:,:) = aux2(k_u_nc_N) + s_u_nc.*(aux2(k_u_nc_N+1)-aux2(k_u_nc_N));
                aux2 = squeeze(V_e_disab(i,j,:,:,end+retire-t+1));
                EV_ued_aux(i,j,:,:) = aux2(k_u_nc_N) + s_u_nc.*(aux2(k_u_nc_N+1)-aux2(k_u_nc_N));
            end
        end
        EV_uu = survival(end-t,1)*((1-tran_hd(end+retire-t+1))*EV_uu_aux+tran_hd(end+retire-t+1)*EV_uud_aux)+aux_beq;
        EV_ue = survival(end-t,1)*((1-tran_hd(end+retire-t+1))*EV_ue_aux+tran_hd(end+retire-t+1)*EV_ued_aux)+aux_beq;
        EV_uud = survival(end-t,1)*((1-tran_dh(end+retire-t+1))*EV_uud_aux+tran_dh(end+retire-t+1)*EV_uu_aux)+aux_beq;
        EV_ued = survival(end-t,1)*((1-tran_dh(end+retire-t+1))*EV_ued_aux+tran_dh(end+retire-t+1)*EV_ue_aux)+aux_beq;
        
        
        %employment choices non-employed
        echoice_u(:,:,:,:,end+retire-t) = EV_ue>=EV_uu; %non-disabled
        echoice_u_disab(:,:,:,:,end+retire-t) = EV_ued>=EV_uud; %disabled
        upper_u = max(EV_ue,EV_uu);
        upper_ud = max(EV_ued,EV_uud);
        
        %productivity transition non-employed
        U_n = expected(EV_uu,PI_u,N,na,nh,nc); %value of no offer
        U_f = expected(upper_u,PI_u,N,na,nh,nc); %value of offer
        U_nd = expected(EV_uud,PI_u,N,na,nh,nc); %value of no offer disabled
        U_fd = expected(upper_ud,PI_u,N,na,nh,nc); %value of offer disabled
        
        %employment choice employed
        echoice_e(:,:,:,:,end+retire-t) = EV_ee>=EV_euj; %non-disabled
        upper_e = max(EV_ee,EV_euj);
        echoice_e_disab(:,:,:,:,end+retire-t) = EV_eed>=EV_eujd; %disabled
        upper_ed = max(EV_eed,EV_eujd);
        
        %productivity transition employed
        E_e = expected(upper_e,PI_e,N,na,nh,nc); %value of no destruction
        E_ed = expected(upper_ed,PI_e,N,na,nh,nc); %value of no destruction disabled
        E_u = expected(EV_euj,PI_e,N,na,nh,nc); %value of destruction
        E_ud = expected(EV_eujd,PI_e,N,na,nh,nc); %value of destruction disabled
        
        %continuation value employment
        EVE = beta*((1-destruct_health(end+retire-t,1))*E_e+destruct_health(end+retire-t,1)*E_u);
        EVEd = beta*((1-destruct_disab(end+retire-t,1))*E_ed+destruct_disab(end+retire-t,1)*E_ud);
        
        %create auxilary variables for employment, non-disabled
        egl_w_aux = repmat(reshape(egl_w(:,life-t),[1 N]),[nc 1]);
        egl_w_aux = egl_w_aux(:);
        I_aux = repmat(reshape(I.w(:,life-t),[1 N]),[nc 1]);
        I_aux = I_aux(:);
        S_aux = repmat(reshape(S.w(:,life-t),[1 N]),[nc 1]);
        S_aux = S_aux(:);
        low_max = cut_prime(1); 
        EVE_aux = reshape(EVE,[na nh nc*N]);
        V_aux = zeros(na,nh,nc*N);
        policy_a_aux = zeros(na,nh,nc*N);
        policy_h_aux = zeros(na,nh,nc*N);
        policy_s_aux = zeros(na,nh,nc*N);
        
        %value function employed, non-disabled
        parfor j = 1:nc*N
            if egl_w_aux(j,1)
                
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j),policy_s_aux(:,:,j)] =...
                    asset_test_off_mex(EVE_aux(:,:,j),agrid,aprime,int32(low_max),abar_aux,int32(na),int32(nap)...
                    ,I_aux(j,1),phi,R,S_aux(j,1),gamma,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,stigma,int32(abound),s_aprime);
                
            else
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j)] =...
                    asset_no_off_mex(EVE_aux(:,:,j),agrid,aprime,int32(na),int32(nap),I_aux(j,1),R,gamma,phi,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,int32(abound),s_aprime);
            end
        end
        V_e(:,:,:,:,life-t) = reshape(V_aux,[na nh nc N]);
        policy_a_e(:,:,:,:,life-t) = reshape(policy_a_aux,[na nh nc N]);
        policy_h_e(:,:,:,:,life-t) = reshape(policy_h_aux,[na nh nc N]);
        policy_s_e(:,:,:,:,life-t) = reshape(policy_s_aux,[na nh nc N]);
        
        
        
        %create auxilary variables for employment disabled
        egl_w_aux = repmat(reshape(egl_wd(:,life-t),[1 N]),[nc 1]);
        egl_w_aux = egl_w_aux(:);
        I_aux = repmat(reshape(I.wd(:,life-t),[1 N]),[nc 1]);
        I_aux = I_aux(:);
        S_aux = repmat(reshape(S.w_d(:,life-t),[1 N]),[nc 1]);
        S_aux = S_aux(:);
        low_max = cut_prime(1); 
        EVE_aux = reshape(EVEd,[na nh nc*N]);
        V_aux = zeros(na,nh,nc*N);
        policy_a_aux = zeros(na,nh,nc*N);
        policy_h_aux = zeros(na,nh,nc*N);
        policy_s_aux = zeros(na,nh,nc*N);
        
        %value function employed, disabled
        parfor j = 1:nc*N
            if egl_w_aux(j,1)
                
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j),policy_s_aux(:,:,j)] =...
                    asset_test_off_mex(EVE_aux(:,:,j),agrid,aprime,int32(low_max),abar_aux,int32(na),int32(nap)...
                    ,I_aux(j,1),phi,R,S_aux(j,1),gamma,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,stigma,int32(abound),s_aprime);
                
            else
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j)] =...
                    asset_no_off_mex(EVE_aux(:,:,j),agrid,aprime,int32(na),int32(nap),I_aux(j,1),R,gamma,phi,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,int32(abound),s_aprime);
            end
        end
        V_e_disab(:,:,:,:,life-t) = reshape(V_aux,[na nh nc N]);
        policy_a_e_disab(:,:,:,:,life-t) = reshape(policy_a_aux,[na nh nc N]);
        policy_h_e_disab(:,:,:,:,life-t) = reshape(policy_h_aux,[na nh nc N]);
        policy_s_e_disab(:,:,:,:,life-t) = reshape(policy_s_aux,[na nh nc N]);
        
        
        %continuation value non-employment
        EVU = beta*((1-offer_health(life-t))*U_n+offer_health(life-t)*U_f);
        EVUd = beta*((1-offer_disab(life-t))*U_nd+offer_disab(life-t)*U_fd);
        
        %auxilary for long term non-employment
        low_max = cut_prime(1);
        S_aux = S.u(life-t);
        S_aux = S_aux(:);
        egl_b_aux = egl_b(life-t);
        I_aux = I.u(life-t);
        EVU_aux = reshape(EVU,[na nh nc*N]);
        V_aux = zeros(na,nh,nc*N);
        policy_a_aux = zeros(na,nh,nc*N);
        policy_h_aux = zeros(na,nh,nc*N);
        policy_s_aux = zeros(na,nh,nc*N);
        
        %value functionnon- employed, non-disabled
        parfor j = 1:nc*N
            if egl_b_aux
                
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j),policy_s_aux(:,:,j)] =...
                    asset_test_off_mex(EVU_aux(:,:,j),agrid,aprime,int32(low_max),abar_aux,int32(na),int32(nap)...
                    ,I_aux,0,R,S_aux,gamma,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,stigma,int32(abound),s_aprime);
                
                
            else
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j)] =...
                    asset_no_off_mex(EVU_aux(:,:,j),agrid,aprime,int32(na),int32(nap),I_aux,R,gamma,0,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,int32(abound),s_aprime);
                
            end
            
        end
        V_u(:,:,:,:,life-t) = reshape(V_aux,[na nh nc N]);
        policy_a_u(:,:,:,:,life-t) = reshape(policy_a_aux,[na nh nc N]);
        policy_h_u(:,:,:,:,life-t) = reshape(policy_h_aux,[na nh nc N]);
        policy_s_u(:,:,:,:,life-t) = reshape(policy_s_aux,[na nh nc N]);
        
        
        %auxilary variables for non-employed disabled
        EVU_aux = reshape(EVUd,[na nh nc*N]);
        V_aux = zeros(na,nh,nc*N);
        policy_a_aux = zeros(na,nh,nc*N);
        policy_h_aux = zeros(na,nh,nc*N);
        policy_s_aux = zeros(na,nh,nc*N);
        
        %value functionnon non-employed, disabled
        parfor j = 1:nc*N
            if egl_b_aux
                
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j),policy_s_aux(:,:,j)] =...
                    asset_test_off_mex(EVU_aux(:,:,j),agrid,aprime,int32(low_max),abar_aux,int32(na),int32(nap)...
                    ,I_aux,0,R,S_aux,gamma,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,stigma,int32(abound),s_aprime);
                
            else
                [V_aux(:,:,j),policy_a_aux(:,:,j),policy_h_aux(:,:,j)] =...
                    asset_no_off_mex(EVU_aux(:,:,j),agrid,aprime,int32(na),int32(nap),I_aux,R,gamma,0,health_aux,hh_scale_aux,hgrid,int32(nh),alpha,hbar,xi,int32(abound),s_aprime);
                
            end
            
        end
        V_u_disab(:,:,:,:,life-t) = reshape(V_aux,[na nh nc N]);
        policy_a_u_disab(:,:,:,:,life-t) = reshape(policy_a_aux,[na nh nc N]);
        policy_h_u_disab(:,:,:,:,life-t) = reshape(policy_h_aux,[na nh nc N]);
        policy_s_u_disab(:,:,:,:,life-t) = reshape(policy_s_aux,[na nh nc N]);
        
        t = t+1;
        
    end
    
    toc
    
    poolobj = gcp('nocreate');
    delete(poolobj);
    
    V_e = V_e(:,:,:,:,1);
    V_e_disab = V_e_disab(:,:,:,:,1);
    V_u = V_u(:,:,:,:,1);
    V_u_disab = V_u_disab(:,:,:,:,1);
    
    clear EV EV_ee EV_ee_aux EV_eed EV_eed_aux EV_euj EV_euj1 EV_euj1_aux EV_euj2 EV_euj2_aux EV_euj_aux EV_eujd EV_eujd1
    clear EV_eujd1_aux EV_eujd2 EV_eujd2_aux EV_eujd_aux EV_ue EV_ue_aux EV_ued EV_ued_aux EV_uu EV_uu_aux EV_uud EV_uud_aux
    clear EVD EVD_aux EVE EVE_aux EVEd EVU EVU_aux EVUd EVUd_aux EVUJ EVUJ_aux EVUJd upper1 upper2 upper_e upper_ed upper_u upper_ud upperd1 upperd2
    clear V_aux V_beq V_uj V_uj_disab V_d V_r transfer
    %% initial conditions
    
    
    %distribution of financial assets of the employed
    ini.emp.all(:,1) = max(ini.emp.all(:,1),agrid(1));
    ini.emp.all(:,1) = min(ini.emp.all(:,1),agrid(end));
    [~,index] = histc(min(max(agrid),ini.emp.all(:,1)),agrid); %find position of data point on agrid
    index(ini.emp.all(:,1)<agrid(1)) = 1;
    index(ini.emp.all(:,1)>=agrid(end)) = na-1;
    s = (ini.emp.all(:,1) - agrid(index))./adiff(index);
    initial_asset_l = zeros(na,1);
    for i = 1:length(index)
        initial_asset_l(index(i,1),1) = initial_asset_l(index(i,1),1)+(1-s(i))*ini.emp.all(i,2);
        initial_asset_l(index(i,1)+1,1) = initial_asset_l(index(i,1)+1,1)+s(i)*ini.emp.all(i,2);
    end
    initial_asset.w = initial_asset_l./sum(initial_asset_l);
    %distribution of financial assets of the non-employed (mean in the data)
    initial_asset.u = zeros(na,1);
    index = sum(ini.une.all(1)>=agrid);
    s = (ini.une.all(1) - agrid(index))./adiff(index);
    initial_asset.u(index,1) = (1-s);
    initial_asset.u(index+1,1) = s;
    %distribution of durables of the employed
    ini.emp.all(:,1) = max(ini.emp.all(:,3),hgrid(1));
    ini.emp.all(:,1) = min(ini.emp.all(:,3),hgrid(end));
    [~,index] = histc(min(max(hgrid),ini.emp.all(:,3)),hgrid); %find position of data point on hgrid
    index(ini.emp.all(:,3)<hgrid(1)) = 1;
    index(ini.emp.all(:,3)>=hgrid(end)) = nh-1;
    s = (ini.emp.all(:,3) - hgrid(index))./hdiff(index);
    initial_asset_l = zeros(nh,1);
    for i = 1:length(index)
        initial_asset_l(index(i,1),1) = initial_asset_l(index(i,1),1)+(1-s(i))*ini.emp.all(i,4);
        initial_asset_l(index(i,1)+1,1) = initial_asset_l(index(i,1)+1,1)+s(i)*ini.emp.all(i,4);
    end
    initial_asset.w_h = initial_asset_l./sum(initial_asset_l);
    %distribution of durables of the non-employed (mean in the data)
    initial_asset.u_h = zeros(nh,1);
    index = sum(ini.une.all(2)>=hgrid);
    s = (ini.une.all(2) - hgrid(index))./hdiff(index);
    initial_asset.u_h(index,1) = (1-s);
    initial_asset.u_h(index+1,1) = s;
    
    %% Update the size of the welfare state
    
    H = 65000; %number of households to simulate
    T = work+retire;
    
    %change dimensions of policies for quicker simulation
    policy_a_r = permute(policy_a_r,[1 3 2 4]);
    policy_h_r = permute(policy_h_r,[1 3 2 4]);
    policy_a_e = permute(policy_a_e,[1 3 4 2 5]);
    policy_h_e = permute(policy_h_e,[1 3 4 2 5]);
    policy_a_u = permute(policy_a_u,[1 3 4 2 5]);
    policy_h_u = permute(policy_h_u,[1 3 4 2 5]);
    policy_a_uj = permute(policy_a_uj,[1 3 4 2 5]);
    policy_h_uj = permute(policy_h_uj,[1 3 4 2 5]);
    policy_s_uj = permute(policy_s_uj,[1 3 4 2 5]);
    policy_a_d = permute(policy_a_d,[1 3 2 4]);
    policy_h_d = permute(policy_h_d,[1 3 2 4]);
    echoice_e = permute(echoice_e,[1 3 4 2 5]);
    echoice_u = permute(echoice_u,[1 3 4 2 5]);
    d_pol = permute(d_pol,[1 3 4 2 5]);
    echoice_e2 = permute(echoice_e2,[1 3 4 2 5]);
    policy_s_e = permute(policy_s_e,[1 3 4 2 5]);
    policy_s_u = permute(policy_s_u,[1 3 4 2 5]);
    policy_s_d = permute(policy_s_d,[1 3 2 4]);
    policy_s_r = permute(policy_s_r,[1 3 2 4]);
    policy_a_e_disab = permute(policy_a_e_disab,[1 3 4 2 5]);
    policy_a_u_disab = permute(policy_a_u_disab,[1 3 4 2 5]);
    policy_a_uj_disab = permute(policy_a_uj_disab,[1 3 4 2 5]);
    policy_h_u_disab = permute(policy_h_u_disab,[1 3 4 2 5]);
    policy_h_e_disab = permute(policy_h_e_disab,[1 3 4 2 5]);
    policy_h_uj_disab = permute(policy_h_uj_disab,[1 3 4 2 5]);
    policy_s_e_disab = permute(policy_s_e_disab,[1 3 4 2 5]);
    policy_s_u_disab = permute(policy_s_u_disab,[1 3 4 2 5]);
    policy_s_uj_disab = permute(policy_s_uj_disab,[1 3 4 2 5]);
    echoice_e2_disab = permute(echoice_e2_disab,[1 3 4 2 5]);
    echoice_e_disab = permute(echoice_e_disab,[1 3 4 2 5]);
    echoice_u_disab = permute(echoice_u_disab,[1 3 4 2 5]);
    d_pol_disab = permute(d_pol_disab,[1 3 4 2 5]);
    
    %keep baseline values in memory  
    w_u_old = w_u_original;
    w_retire_old = w_retire_original;
    w_disab_old = w_disab_original;
    w_old = w_original;
    wd_old = wd_original;
    
    S.w_old = Result_all.S.w;
    S.w_d_old = Result_all.S.w_d;
    S.d_old = Result_all.S.d;
    S.r_old = Result_all.S.r;
    S.u_old = Result_all.S.u;
    
  
    crit.costs = 100000000
    count = 0;
    
    %shocks
    randn('seed',88596);
    prod_shock = sigma*randn(H,work);
    rand('seed',21424);
    randn('seed',168424);
    sim_initial_asset = rand(H,1);
    sim_initial_house = rand(H,1);
    sim_initial_emp = rand(H,1);
    sim_initial_dis = rand(H,1);
    sim_destruct = rand(H,work);
    sim_offer = rand(H,work);
    sim_dis = rand(H,work);
    sim_disab = rand(H,work);
    sim_health = rand(H,work);
    sim_death = rand(H,T);
    [sim_initial_perm,born_prob] = get_born(ini_dist,wgross(:,1),squeeze(echoice_e(20,1,1,:,1)),H,pmin,pmax);  
    
    while crit.costs > tol.costs && count<10
        count = count+1;
        
        [panel,costs] = simulation_mex(int32(H),int32(T),int32(work),int32(retire),int32(disab),int32(old),int32(N),int32(na),int32(nc),amax,R,amin,destruct_health,offer_health,destruct_disab,offer_disab,born_prob...
            ,urate_ini,disab_ini,agrid,grid_nc,initial_asset.w,initial_asset.u,pgrid,pgrid_d,policy_a_e,policy_a_u,policy_a_uj,policy_a_d,policy_a_r,policy_a_e_disab,policy_a_u_disab,policy_a_uj_disab...
            ,echoice_e,echoice_u,echoice_e_disab,echoice_u_disab,echoice_e2,echoice_e2_disab,d_pol,d_pol_disab,S.w,S.u,S.d,S.r,S.w_d,w,b,w_disab,w_retire,wd,wgross_r,wgross_d,wdgross...
            ,abar,abar2,survival,w_u,health_life,wgross,egl_w,egl_b,egl_d,egl_r,egl_wd,psi,psi_d,mu,mu2,delta1,delta2,disab_dis,pmin,pmax,prod_shock,hgrid...
            ,int32(nh),policy_h_e,policy_h_u,policy_h_uj,policy_h_d,policy_h_r,policy_h_e_disab,policy_h_u_disab,policy_h_uj_disab,sim_initial_asset...
            ,sim_initial_perm,sim_initial_emp,sim_destruct,sim_offer,sim_death,sim_dis,sim_disab,sim_health,xi,d_cut,initial_asset.w_h,initial_asset.u_h...
            ,sim_initial_house,sim_initial_dis,tran_dh,tran_hd,policy_s_e,policy_s_u,policy_s_uj,policy_s_d,policy_s_r,policy_s_e_disab...
            ,policy_s_u_disab,policy_s_uj_disab,ybar_y,ybar_o,ybar_ss);
        
        crit.costs = max(abs(costs-Result_all.costs)./Result_all.costs)
        
        if count==1 && crit.costs<crit.all
            crit.all = 0;
        else
            if abs(abs(costs(1)-Result_all.costs(1))./Result_all.costs(1))>0.002
                up = min(0.001,abs(costs(1)-Result_all.costs(1))/10000000000);
                if costs(1)>Result_all.costs(1) %lower taxes needed
                    rr.tau = rr.tau+up;
                else
                    rr.tau = rr.tau-up;
                end
            end
            if abs(abs(costs(2)-Result_all.costs(2))./Result_all.costs(2))>0.002 %rr.S=0.973  -7.4e6
                up = min(0.001,abs(costs(2)-Result_all.costs(2))/500000000);
                if costs(2)>Result_all.costs(2) %lower transfers needed
                    rr.S = rr.S-up;
                else
                    rr.S = rr.S+up;
                end
            end
            if abs(abs(costs(3)-Result_all.costs(3))./Result_all.costs(3))>0.002
                up = min(0.001,abs(costs(3)-Result_all.costs(3))/500000000);
                if costs(3)>Result_all.costs(3) %lower benefits needed
                    rr.b = rr.b-up;
                else
                    rr.b = rr.b+up;
                end
            end
            if abs(abs(costs(4)-Result_all.costs(4))./Result_all.costs(4))>0.002 %rr.d=0.55   1.0e7
                up = min(0.001,abs(costs(4)-Result_all.costs(4))/50000000);
                if costs(4)>Result_all.costs(4) %lower disab benefits needed
                    rr.d = rr.d-up;
                else
                    rr.d = rr.d+up;
                end
            end
            if abs(abs(costs(5)-Result_all.costs(5))./Result_all.costs(5))>0.002
                up = min(0.001,abs(costs(5)-Result_all.costs(5))/1000000000);
                if costs(5)>Result_all.costs(5) %lower disab benefits needed
                    rr.ss = rr.ss-up;
                else
                    rr.ss = rr.ss+up;
                end
            end
            
            
            %update unemployment benefits
            w_u(:,1)=rr.b*w_u_old(:,1);
            w_u_all = w_u(:,1)+w_u_old(:,2);
            
            
            %update gross disability/retirement income
            aux = rr.ss*w_retire_old;
            wgross_r = aux+pensions;
            wgross_d = rr.d*w_disab_old;
            wgross = w_old;
            wdgross = wd_old;
            
            %net income
            for j = 1:N
                for t = 1:work
                    if hh_size(t)<1 %no kids     3
                        if wgross(j,t)<=35800/3%less than 35800
                            w(j,t) = 0.85*wgross(j,t);
                        elseif wgross(j,t)>35800/3 && wgross(j,t)<=86500/3%less than 86500
                            w(j,t) = wgross(j,t)-1790-0.28*(wgross(j,t)-(35800/3));
                        else %more than 86500
                            w(j,t) = wgross(j,t)-6522-0.31*(wgross(j,t)-(86500/3));
                        end
                        if wdgross(j,t)<=35800/3%less than 35800
                            wd(j,t) = 0.85*wdgross(j,t);
                        elseif wdgross(j,t)>35800/3 && wdgross(j,t)<=86500/3%less than 86500
                            wd(j,t) = wdgross(j,t)-1790-0.28*(wdgross(j,t)-(35800/3));
                        else %more than 86500
                            wd(j,t) = wdgross(j,t)-6522-0.31*(wdgross(j,t)-(86500/3));
                        end
                    else
                        if wgross(j,t)<=7750/3%less than 7750
                            w(j,t) = 1.035*wgross(j,t);
                        elseif wgross(j,t)>7750/3 && wgross(j,t)<=12200/3%less than 12200
                            w(j,t) = 478+0.85*wgross(j,t);
                        elseif wgross(j,t)>12200/3 && wgross(j,t)<=23050/3 %less than 23050
                            w(j,t) = 478+0.718*wgross(j,t);
                        elseif wgross(j,t)>23050/3 && wgross(j,t)<=35800/3 %less than 35800
                            w(j,t) = 0.85*wgross(j,t);
                        elseif wgross(j,t)>35800/3 && wgross(j,t)<=86500/3%less than 86500
                            w(j,t) = wgross(j,t)-1790-0.28*(wgross(j,t)-(35800/3));
                        else %more than 86500
                            w(j,t) = wgross(j,t)-6522-0.31*(wgross(j,t)-(86500/3));
                        end
                        
                        if wdgross(j,t)<=7750/3%less than 7750
                            wd(j,t) = 1.035*wdgross(j,t);
                        elseif wdgross(j,t)>7750/3 && wdgross(j,t)<=12200/3%less than 12200
                            wd(j,t) = 478+0.85*wdgross(j,t);
                        elseif wdgross(j,t)>12200/3 && wdgross(j,t)<=23050/3 %less than 23050
                            wd(j,t) = 478+0.718*wdgross(j,t);
                        elseif wdgross(j,t)>23050/3 && wdgross(j,t)<=35800/3 %less than 35800
                            wd(j,t) = 0.85*wdgross(j,t);
                        elseif wdgross(j,t)>35800/3 && wdgross(j,t)<=86500/3%less than 86500
                            wd(j,t) = wdgross(j,t)-1790-0.28*(wdgross(j,t)-(35800/3));
                        else %more than 86500
                            wd(j,t) = wdgross(j,t)-6522-0.31*(wdgross(j,t)-(86500/3));
                        end
                    end
                end
            end
            for i = 1:nc
                for t = 1:retire
                    if wgross_r(i,t)/2<=35800/3%less than 35800
                        w_retire(i,t) = wgross_r(i,t)-0.15*wgross_r(i,t)/2;
                    elseif wgross_r(i,t)/2>35800/3 && wgross_r(i,t)/2<=86500/3%less than 86500
                        w_retire(i,t) = wgross_r(i,t)-(0.15*35800/6)-0.28*(wgross_r(i,t)/2-(35800/6));
                    else %more than 86500
                        w_retire(i,t) = wgross_r(i,t)-(0.15*35800/6+0.28*(86500/6-0.15*35800/6))-0.31*(wgross_r(i,t)-(86500/6));
                    end
                end
                for t = 1:disab
                    if wgross_d(i,t)/2<=35800/3%less than 35800
                        w_disab(i,t) = wgross_d(i,t)-0.15*wgross_d(i,t)/2;
                    elseif wgross_d(i,t)/2>35800/3 && wgross_d(i,t)/2<=86500/3%less than 86500
                        w_disab(i,t) = wgross_d(i,t)-(0.15*35800/6)-0.28*(wgross_d(i,t)/2-(35800/6));
                    else %more than 86500
                        w_disab(i,t) = wgross_d(i,t)-(0.15*35800/6+0.28*(86500/6-0.15*35800/6))-0.31*(wgross_d(i,t)-(86500/6));
                    end
                end
            end
            
            %wage for interpolation
            wL = repmat(reshape(w,[1 N work]),[nc 1 1]);
            grid_ncL = repmat(grid_nc,[1 N]);
            
            %update net wages
            w_retire = w_retire*rr.tau;
            w_disab = w_disab*rr.tau;
            w = w*rr.tau;
            wd = wd*rr.tau;
            
            %egligible to support
            egl_w = zeros(N,work); %number of income states where hh is eglidgible to income support
            for t = 1:work
                for j = 1:N
                    egl_w(j,t) = wgross(j,t)<ybar_y(t,1); 
                end
            end
            
            egl_wd = zeros(N,work); %number of income states where hh is eglidgible to income support
            for t = 1:work
                for j = 1:N
                    egl_wd(j,t) = wdgross(j,t)<ybar_y(t,1);
                end
            end
            
            egl_b = zeros(work,1); %no benefits
            for t = 1:work
                egl_b(t) = (w_u_all(t,1))<ybar_y(t,1);
            end
            
            egl_d = zeros(nc,disab); %number of income states where hh is eglidgible to income support
            for t = 1:disab
                for i = 1:nc
                    egl_d(i,t) = wgross_d(i,t)<ybar_o(work-disab+t,1); 
                end
            end
            
            egl_dss = zeros(nc,disab); %number of income states where hh is eglidgible to ss income support
            for t = 1:disab
                for i = 1:nc
                    egl_dss(i,t) = wgross_d(i,t)<ybar_ss(work-disab+t,1); 
                end
            end
            
            egl_r = zeros(nc,retire); %number of income states where hh is eglidgible to income support
            for t = 1:retire
                for i = 1:nc
                    egl_r(i,t) = wgross_r(i,t)<ybar_o(work+t,1); 
                end
            end
            
            egl_rss = zeros(nc,retire); %number of income states where hh is eglidgible to ss income support
            for t = 1:retire
                for i = 1:nc
                    egl_rss(i,t) = wgross_r(i,t)<ybar_ss(work+t,1);
                end
            end
            
            S = get_transfer(wgross,wdgross,wgross_r,wgross_d,w_u_all,b,N,nc,work,retire,disab,egl_w,egl_wd,egl_r,egl_rss,egl_d,egl_dss,egl_b,hh_size);
            
            S.w = S.w*rr.S;
            S.w_d = S.w_d*rr.S;
            S.d = S.d*rr.S;
            S.r = S.r*rr.S;
            S.u = S.u*rr.S;
            
            %assure that total income is increasing in productivity (have to do it twice)
            for i = 1:2
                aux = w+S.w;
                aux2 = [zeros(1,work); diff(aux,1)];
                index = aux2<0;
                S.w(index) = S.w(index)-aux2(index);
            end
            
            egl_w(S.w>0) = 1; %number of income states where hh is eglidgible to income support
            
            for i = 1:2
                aux = wd+S.w_d;
                aux2 = [zeros(1,work); diff(aux,1)];
                index = aux2<0;
                S.w_d(index) = S.w_d(index)-aux2(index);
            end
            
            egl_wd(S.w_d>0) = 1; %number of income states where hh is eglidgible to income support
            
            %assure that total income is increasing in accumulated earnings (have to do it twice)
            for i = 1:2
                aux = w_disab+S.d;
                aux2 = [zeros(1,disab); diff(aux,1)];
                index = aux2<0;
                S.d(index) = S.d(index)-aux2(index);
            end
            egl_d(S.d>0) = 1;
            
            %assure that total income is increasing in accumulated earnings (have to do it twice)
            for i = 1:2
                aux = w_retire+S.r;
                aux2 = [zeros(1,retire); diff(aux,1)];
                index = aux2<0;
                S.r(index) = S.r(index)-aux2(index);
            end
            egl_r(S.r>0) = 1;
            
        end
        
    end
    [abs(costs(1)-Result_all.costs(1));
        abs(costs(2)-Result_all.costs(2));
        abs(costs(3)-Result_all.costs(3));
        abs(costs(4)-Result_all.costs(4));
        abs(costs(5)-Result_all.costs(5))]
end
rr_5000 = rr;
save('rr_5000.mat','rr_5000') %save scaling parameter



%% utility of an unborn (na,nh,nc,N)
aux1 = repmat(initial_asset.u,[1 nh N]);
aux2 = repmat(initial_asset.w,[1 nh N]);
aux3 = repmat(reshape(initial_asset.u_h,[1 nh 1]),[na 1 N]);
aux4 = repmat(reshape(initial_asset.w_h,[1 nh 1]),[na 1 N]);
aux5 = repmat(reshape(born_prob,[1 1 N]),[na nh 1]);
aux1 = aux1.*aux3.*aux5;
aux2 = aux2.*aux4.*aux5;
VbarU = disab_ini*aux1.*squeeze(V_u_disab(:,:,1,:))+(1-disab_ini)*aux1.*squeeze(V_u(:,:,1,:));
VbarE = disab_ini*aux2.*squeeze(V_e_disab(:,:,1,:))+(1-disab_ini)*aux2.*squeeze(V_e(:,:,1,:));
Vbar = sum(sum(sum(urate_ini.*VbarU+(1-urate_ini).*VbarE)));
result_5000.Vbar = Vbar;


save('result_5000.mat','result_5000')