local PATH "F:\Insurance\Revision\Numerics\Clean\Stata"

cd "`PATH'"
use "F:\Insurance\Revision\Numerics\Clean\Stata\aggregated", clear

drop if emp==0
drop if di==1
keep if age>24
drop if age >= 65


*distribution of earnings
preserve
order hearn_inc age hwgt
keep hearn_inc age hwgt
outsheet using earn_dist, comma nolabel nonames replace
restore

*obtain unconditional mean initial earnings 
gen aux=log(hearn_inc)
egen ini_earn_unc = mean(aux) if age==25 
egen ini_earn2 = max(ini_earn_unc)
replace ini_earn_unc=ini_earn2
drop ini_earn2


*obtain mean initial earnings conditional on non-disability
egen ini_earn = mean(hearn_inc) if age==25 & disab==0
egen ini_earn2 = max(ini_earn)
replace ini_earn=ini_earn2
drop ini_earn2
replace ini_earn=log(ini_earn)

*recode
replace sex=0 if sex==1
replace race = 0 if race==1
replace hmetro = 0 if hmetro==1
replace ms = 0 if ms==1
replace educ = 0 if educ==4

*crate age-bins
gen age_bin=1 if age>=25 & age<=27
replace age_bin=2 if age>=28 & age<=30
replace age_bin=3 if age>=31 & age<=33
replace age_bin=4 if age>=34 & age<=36
replace age_bin=5 if age>=37 & age<=39
replace age_bin=6 if age>=40 & age<=42
replace age_bin=7 if age>=43 & age<=45
replace age_bin=8 if age>=46 & age<=48
replace age_bin=9 if age>=49 & age<=51
replace age_bin=10 if age>=52 & age<=54
replace age_bin=11 if age>=55 & age<=57
replace age_bin=12 if age>=58 & age<=61
replace age_bin=13 if age>=62 & age<=64

*obtain residuals
gen learn = log(hearn_inc)
xi: reg learn i.age i.disab i.year i.sex i.race i.hmetro i.ms i.educ i.hstate [aweight=hwgt]




mat beta=e(b)
svmat double beta, name(matcol)
keep beta* ini_earn ini_earn_unc

drop beta_Iyear* beta_Isex* beta_Irace* beta_Ihmetro* beta_Ims* beta_Ieduc* beta_Ihstate*
drop if beta_Iage_26==.

*generate intercepts (age 25)
replace beta_cons = ini_earn_unc
rename beta_cons beta_nodisab


*add intercept to age profiles
forval i=26(1)64{
replace beta_Iage_`i'  = beta_nodisab+beta_Iage_`i'
}

drop ini_earn
order beta_nodisab beta_Iage_* beta_Idisab_1
outsheet using profiles, comma nolabel nonames replace

