set mem 700m

local PATH "F:\Insurance\Revision\Numerics\Clean\Stata"


cd "`PATH'"

use 90_panel.dta, clear

sort suid addid panel
quiet merge suid addid panel using "91_panel.dta"
tab _merge
drop _merge
sort suid addid panel
quiet merge suid addid panel using "92_panel.dta"
tab _merge
drop _merge
sort suid addid panel
quiet merge suid addid panel using "93_panel.dta"
tab _merge
drop _merge

drop enrgy_r enrgy_a s13amt prop other tran mst_rgc month2 hother
destring addid, replace

*define education
gen educ=1 if grade<=12 
replace educ=2 if grade==12 & grd_cmp==1
replace educ=3 if grade>=13 & grade<=24
replace educ=4 if grade==24 & grd_cmp==1
replace educ=4 if grade>24 
drop grd_cmp grade

*define selfincome
gen selfinc = selfinc1+selfinc2
drop selfinc1 selfinc2

*define quarter
gen quarter = 1 if month<4
replace quarter = 2 if month>3 & month<7
replace quarter = 3 if month>6 & month<10
replace quarter = 4 if month>9

*compute share of ssi individuals that are disabled
preserve
keep if age>=50
egen pid = group(panel suid entry pnum)
gen index1 = disage>1
gen index2 = socsr1==2
gen index3 = socsr2==2
gen index = (index1+index2+index3)>0
sort pid wave
by pid: gen di = sum(index)
replace di=1 if di>0
keep if di>0
sum disab
restore

*********************************aggregate to wave**************************************
*keep individual only when complete wave is observed 
egen id = group(panel suid addid)
egen pid = group(panel suid entry pnum)
egen obs_wave = sum(suid!="."), by(pid wave)
replace obs_wave = obs_wave==4
drop if obs_wave ==0
drop obs_wave

*start aggregation
bysort pid wave: egen aux = mode(esr), missing minmode 
replace esr = aux
drop aux
bysort pid wave: egen aux = mode(hrefper), missing minmode  
replace hrefper = aux
drop aux
bysort pid wave: egen aux = mode(hnp), missing minmode  
replace hnp = aux
drop aux
bysort pid wave: egen aux = mode(hwgt), missing minmode  
replace hwgt = aux
drop aux
bysort pid wave: egen aux = mode(hmetro), missing minmode  
replace hmetro = aux
drop aux
bysort pid wave: egen aux = mode(hpov), missing minmode  
replace hpov = aux
drop aux
bysort pid wave: egen aux = mode(empled), missing minmode  
replace empled = aux
drop aux
bysort pid wave: egen aux = mode(retird), missing minmode  
replace retird = aux
drop aux
bysort pid wave: egen aux = mode(age), missing minmode  
replace age = aux
drop aux
bysort pid wave: egen aux = mode(ms), missing minmode  
replace ms = aux
drop aux
bysort pid wave: egen aux = mode(disab), missing minmode  
replace disab = aux
drop aux         
bysort pid wave: egen aux = mode(class1), missing minmode  
replace class1 = aux
drop aux  		 
bysort pid wave: egen aux = mode(occ31), missing minmode  
replace occ31 = aux
drop aux  			 
bysort pid wave: egen aux = mode(ind31), missing minmode  
replace ind31 = aux
drop aux  		 		 
bysort pid wave: egen aux = mode(hstate), missing minmode  
replace hstate = aux
drop aux		
bysort pid wave: egen aux = mode(year), missing minmode  
replace year = aux
drop aux    
bysort pid wave: egen aux = mode(quarter), missing minmode  
replace quarter = aux
drop aux  
replace hmeans = hmeans==1	
bysort pid wave: egen aux = mode(hmeans), missing minmode  
replace hmeans = aux
drop aux 	  
			    
rename hnoncsh hh_noncsh
rename hsocsec hh_socsec
rename hssi hh_ssi
rename hunemp hh_unemp
rename hafdc hh_afdc 
rename hfdstp hh_fdstp
rename hmeans hh_means

*aggregate to id/wave level
collapse (sum) hh_noncsh hh_socsec hh_ssi hh_unemp hh_afdc hh_fdstp hh_inc hh_ear hh_pro hh_tra hh_oth totinc selfinc earn sose_inc ret_inc pension_p_inc pension_g_inc unemp_inc other_inc prop_inc ssi_inc afdc oth_welfare food wic (first) year quarter suid addid panel entry pnum rot sex race ethnicty socsr1 socsr2 disage esr hrefper hnp hwgt hmetro hh_means hpov empled inaf retird age ms disab class1 occ31 ind31 enrold hstate (median) month pubrnamt (max) educ hh_theq hh_vehcl hh_beq hh_intbk hh_intot hh_stk hh_ore hh_otast hh_ira hh_usdbt medi_exp, by(pid wave)

replace medi_exp = medi_exp*4
replace hpov = hpov/3


*generate household as group of people living together (including movers where composition stays the same)
egen id = group(panel suid addid)
egen id_pid=group(id pid)
sort id_pid wave
by id_pid: gen aux = _n
replace aux=aux==1
replace aux=pid if aux==1
sort id
by id: egen aux2 = sum(aux)
sort pid wave
by pid: gen aux3 = addid[_n]-addid[_n-1]
replace aux3=0 if aux3==.
sort pid wave
gen aux4 = 0
by pid: replace aux4 = addid[_n-1] if aux3>0 & aux2[_n]==aux2[_n-1]
sort id wave
by id: egen aux5=max(aux4)
replace aux5=addid if aux5==0
replace addid=aux5
drop id id_pid aux aux2 aux3 aux4 aux5
*second time
egen id = group(panel suid addid)
egen id_pid=group(id pid)
sort id_pid wave
by id_pid: gen aux = _n
replace aux=aux==1
replace aux=pid if aux==1
sort id
by id: egen aux2 = sum(aux)
sort pid wave
by pid: gen aux3 = addid[_n]-addid[_n-1]
replace aux3=0 if aux3==.
sort pid wave
gen aux4 = 0
by pid: replace aux4 = addid[_n-1] if aux3>0 & aux2[_n]==aux2[_n-1]
sort id wave
by id: egen aux5=max(aux4)
replace aux5=addid if aux5==0
replace addid=aux5
drop id id_pid aux aux2 aux3 aux4 aux5
*3rd time
sort pid wave
egen id = group(panel suid addid)
egen id_pid=group(id pid)
sort id_pid wave
by id_pid: gen aux = _n
replace aux=aux==1
replace aux=pid if aux==1
sort id
by id: egen aux2 = sum(aux)
sort pid wave
by pid: gen aux3 = addid[_n]-addid[_n-1]
replace aux3=0 if aux3==.
sort pid wave
gen aux4 = 0
by pid: replace aux4 = addid[_n-1] if aux3>0 & aux2[_n]==aux2[_n-1]
sort id wave
by id: egen aux5=max(aux4)
replace aux5=addid if aux5==0
replace addid=aux5
drop id id_pid aux aux2 aux3 aux4 aux5


*assign a new person id when household changes (movers)
drop pid
egen pid = group(panel suid entry pnum addid)
egen id = group(panel suid addid)

*generate disability income index based on person with highest ss income in household
bysort pid: egen sose_tot = sum(sose_inc)
bysort id: egen sose_tot_max = max(sose_tot)
replace sose_tot_max=. if sose_tot_max==0
gen index=sose_tot==sose_tot_max 
replace index=0 if sose_tot==0
gen index2 = index
gen index3 = index2
replace index = socsr1 if index==1
replace index2 = socsr2 if index2==1
replace index3 = disage if index3==1
bysort id: egen hh_reason = max(index)
bysort id: egen hh_reason2 = max(index2)
bysort id: egen hh_disage = max(index3)
replace socsr1=hh_reason
replace socsr2=hh_reason2
replace disage = hh_disage
drop index index2 index3 hh_reason hh_reason2 hh_disage
gen index1 = disage>1
gen index2 = socsr1==2
gen index3 = socsr2==2
gen index = (index1+index2+index3)>0
sort id wave
by id: gen di = sum(index)
replace di=1 if di>0
drop index index1 index2 index3 sose_tot sose_tot_max

*compute share of ssi households that are disabled
preserve
keep if age>=50
keep if di>0
sum disab
restore



***********determine reference person****************
*clean reference person: in case of several, keep the first. In case of non, assign the first
bysort id wave: egen index10 = sum(hrefper)
by id wave: egen index11 = min(pid)
replace hrefper=0 if index11!=pid & index10>1
replace hrefper=1 if index11==pid & index10==0
drop index10 index11
*person with most observations as reference person is always reference person (if the same, the one with highest age)
replace hrefper = hrefper == pnum
sort pid
by pid: egen index = sum(hrefper)
sort id
by id: egen index2 = max(index)
replace hrefper = index==index2
drop index index2
sort id pid wave
by id: gen index = pid-pid[_n-1]
replace index = 1 if index ==.
replace index = 1 if index!=0 
replace index = 0 if hrefper==0 
by id: egen index2 = sum(index)
sort pid
by pid: egen age_m = max(age)
sort id
by id: egen id_age_m = max(age)
gen index3 = age_m==id_age_m
replace hrefper = 1 if index2>1 & index3==1
replace hrefper = 0 if index2>1 & index3==0
drop index index2 index3 age_m id_age_m
*find person with highest income. Check whether these are several
gen inc_aux = earn+sose_inc+ret_inc+pension_p_inc+pension_g_inc+unemp_inc  
sort pid 
by pid: egen tot_inc = sum(inc_aux)
sort id
by id: egen max_inc = max(tot_inc)
gen index = tot_inc==max_inc
sort id pid wave
by id: gen index2 = pid-pid[_n-1]
replace index2 = 1 if index2 ==.
replace index2 = 1 if index2!=0 
replace index2 = 0 if index==0 
by id: egen index3 = sum(index2)
*those where several have highest income, assign with the longest observation. 
sort pid wave
by pid: gen max_n = _N
sort id wave
by id: egen index4 = max(max_n) if index==1
replace index4 = index4==max_n
replace index = 0 if index4==0 & index3>1
drop max_n index2 index3 
sort id pid wave
by id: gen index2 = pid-pid[_n-1]
replace index2 = 1 if index2 ==.
replace index2 = 1 if index2!=0 
replace index2 = 0 if index==0 
by id: egen index3 = sum(index2)
*If still equal, take the first observation
by id: egen index5 = min(pid) if index4==1
replace index5 = 0 if index5==.
by id: egen index6 = max(index5) 
replace index = 0 if index6!=pid & index3>1
drop index4 index2 index3
sort id pid wave
by id: gen index2 = pid-pid[_n-1]
replace index2 = 1 if index2 ==.
replace index2 = 1 if index2!=0 
replace index2 = 0 if index==0 
by id: egen index3 = sum(index2)
drop index5 index6 index2 index3
*if alternative ref income has more inc than ref person, she is older than = 25, and she has same numbers of observations change the ref person
gen refper = 0
replace refper=1 if index==1 
gen index2 = refper!=hrefper
gen index3 = 0
replace index3 = 1 if max_inc>tot_inc
sort id
by id: egen index4 = max(index3)
sort pid
by pid: gen index5 = _N if hrefper==1
replace index5 = 0 if index5==.
by pid: gen index6 = _N if refper==1
replace index6 = 0 if index6==.
sort id pid wave
by id: egen index7 = max(index5)
by id: egen index8 = max(index6)
replace index8 = index8>=index7
gen index9 = 0
replace index9=age if refper==1
by id: egen index10 = max(index9)
replace index10 = index10>=25
replace hrefper = refper if index2==1 & index4==1 & index8==1 & index10==1
drop inc_aux tot_inc max_inc refper index index2 index3 index4 index5 index6 index7 index8 index9 index10
*start new household if ref per changes because composition changes (oldest person becomes reference person)
bysort id wave: egen index = sum(hrefper)
replace addid = addid+55 if index==0
drop id pid index
egen id = group(panel suid addid)
egen pid = group(panel suid entry pnum addid)
egen id_q = group(id wave)
sort id
by id: egen index = sum(hrefper)
by id: egen id_age_m = max(age)
sort pid
by pid: egen age_m = max(age)
gen index3 = age_m==id_age_m
replace hrefper = 1 if index3==1 & index==0
drop index index3 id_age_m age_m
sort id wave
by id: gen index = wave-wave[_n-1]
replace index=1 if index==.
replace index = index!=1
by id: gen index2 = sum(index)
replace index2 = index2>0
replace addid = addid+55 if index2==1
drop id
egen id = group(panel suid addid)
drop index index2 
sort id wave
by id: gen index = wave-wave[_n-1]
replace index=1 if index==.
replace index = index!=1
by id: gen index2 = sum(index)
replace index2 = index2>0
replace addid = addid+55 if index2==1
drop id
egen id = group(panel suid addid)
drop index index2


***************agregate means-tested income to household******************
bysort id wave: egen hssi_inc = sum(ssi_inc)
bysort id wave: egen hotherw_inc = sum(oth_welfare)
bysort id wave: egen hafdc_inc = sum(afdc)
bysort id wave: egen hfood_inc = sum(food)
bysort id wave: egen hwic_inc = sum(wic)


*********************************aggregate income to reference person*****************************
bysort id wave: egen hself_inc = sum(selfinc)
bysort id wave: egen htot_inc = sum(totinc)
bysort id wave: egen hearn_inc = sum(earn)
bysort id wave: egen hsose_inc = sum(sose_inc)
bysort id wave: egen hret_inc = sum(ret_inc)
bysort id wave: egen hpension_p_inc = sum(pension_p_inc)
bysort id wave: egen hpension_g_inc = sum(pension_g_inc)
bysort id wave: egen hunemp_inc = sum(unemp_inc)
bysort id wave: egen hother_inc = sum(other_inc)
bysort id wave: egen hprop_inc  = sum(prop_inc)
bysort id wave: egen hmed_exp  = sum(medi_exp)
replace hmed_exp=. if hmed_exp==0 & medi_exp==.
rename hh_inc hh_totinc

gen aux = hret_inc+hsose_inc+hpension_p_inc+hpension_g_inc+hunemp_inc+hother_inc+hearn_inc
replace htot_inc=aux if htot_inc<aux

drop aux

*drop school enroled, too young
keep if hrefper ==1
drop if age<25
drop if age==85
drop if enrold == 1 | enrold == 2
drop enrold

*drop if households has business equity or large selfincome
sort id
by id: egen bus = max(hh_beq)
gen index = hself_inc>500 & hself_inc>1.5*hearn_inc
replace hearn_inc = hself_inc+hearn_inc
by id: egen index2 = max(index)
drop if bus>500 & bus!=.
drop if index2>0
drop bus hh_beq index index2 hself_inc

*drop if household is (was) a public employee 
drop if hpension_g_inc>50 
drop if hret_inc>50 
drop if class1 >2 & class1<7

*reasign id when observation is missing 
sort id wave
by id: gen dtime = wave-wave[_n-1]
replace dtime=1 if dtime==.
replace dtime = dtime>1
by id: gen index=sum(dtime)
egen id2=group(id dtime)
replace id=id2 
drop id2
sort id wave


*********************************define employment, OLF*****************************
gen hh_lincome = hearn_inc+hret_inc+hsose_inc+hpension_p_inc+hpension_g_inc+hunemp_inc
gen emp = 0
replace emp=1 if hearn_inc>hunemp_inc & hearn_inc>=800
replace di=0 if hsose_inc==0
replace di=0 if emp==1
gen lout = esr==8
replace lout=0 if emp==1
replace lout = 1 if di==1

drop hh_noncsh hh_socsec hh_ssi hh_unemp hh_afdc hh_fdstp hh_totinc hh_ear hh_pro hh_tra hh_oth
drop totinc selfinc earn sose_inc ret_inc pension_p_inc pension_g_inc unemp_inc other_inc prop_inc ssi_inc afdc oth_welfare food wic medi_exp


*deflate to 92 Q1
replace year=year+1900 if year<100
sort year quarter
merge year quarter using cpi_quarterly
drop if _merge==2
drop _merge
replace cpi = cpi/138.667
foreach var in hssi_inc hotherw_inc hafdc_inc hfood_inc hwic_inc htot_inc hearn_inc hsose_inc hret_inc hpension_p_inc hpension_g_inc hunemp_inc hother_inc hprop_inc hmed_exp hpov{
replace `var'=`var'/cpi
}


*total net wealth-hh_theq  and  total non-asset income 
gen net_wealth =  hh_intbk + hh_intot + hh_stk + hh_ore + hh_otast + hh_ira - hh_usdbt 
replace hh_theq = hh_theq+hh_vehcl


replace hnp = 7 if hnp>7

save "F:\Insurance\Revision\Numerics\Clean\Stata\aggregated", replace

*****Generate files read-in by Matlab***************

*wealth distribution
preserve
sort id wave
by id: gen eu = emp[_n]-emp[_n-1]
replace eu = 0 if eu!=-1
drop if net_wealth==. 
sort id wave
keep id wave age emp net_wealth hearn_inc hwgt educ eu hh_theq
order id wave age emp net_wealth hearn_inc hwgt educ eu hh_theq
outsheet using asset_dist, comma nolabel nonames replace
restore

*number of persons in household
preserve
gen means = hssi_inc+hotherw_inc+hafdc_inc+hfood_inc+hwic_inc
gen index5 = 1
sort age
collapse(mean) hnp [aw=hwgt], by(age index5)
sort index5 age
keep hnp index5
outsheet using hnp_dist, comma nolabel nonames replace
restore

*moments of disability
preserve
gen means = hssi_inc+hotherw_inc+hafdc_inc+hfood_inc+hwic_inc
sort id wave
keep id hwgt means hnp age disab wave hearn_inc
order id hwgt means hnp age disab wave hearn_inc
outsheet using age_dist, comma nolabel nonames replace
restore

*medical expenditure
preserve
gen means = hssi_inc+hotherw_inc+hafdc_inc+hfood_inc+hwic_inc
keep if hmed_exp!=.
keep if means ==0
sort age
collapse(mean) hmed_exp [aw=hwgt], by(age)
keep hmed_exp
outsheet using hmed_exp, comma nolabel nonames replace
restore

*DI by age
preserve
sort age
collapse(mean) di [aw=hwgt], by(age)
keep di
rename di di_age
outsheet using di_age, comma nolabel nonames replace
restore

*people in DI
preserve
keep if age>=50
keep if age<=65
sort disab
collapse(mean) di [aw=hwgt], by(disab)
keep di
outsheet using di, comma nolabel nonames replace
restore

*unemployment income (non-retired, non-DI receipients)
preserve
sort id wave
keep if age<=64
keep if emp==0 
gen means = hssi_inc+hotherw_inc+hafdc_inc+hfood_inc+hwic_inc
drop if di==1
drop if hssi_inc>hunemp_inc
keep id wave educ age hunemp_inc hearn_inc hwgt 
order id wave educ age hunemp_inc hearn_inc hwgt 
outsheet using unemp_determ, comma nolabel nonames replace
restore

*employment transitions
preserve
gen ret_aux = 0
replace ret_aux=1 if (hret_inc+hpension_p_inc+hpension_g_inc+hsose_inc)>hearn_inc
sort id wave
keep id wave hwgt emp lout age educ ret_aux di disab
order id wave hwgt emp lout age educ ret_aux di disab
outsheet using trans, comma nolabel nonames replace
restore

*means tested transfers
preserve
gen means = hssi_inc+hotherw_inc+hafdc_inc+hfood_inc+hwic_inc
replace htot_inc = htot_inc-means
replace hret_inc = hret_inc+hpension_p_inc+hpension_g_inc+hsose_inc
replace net_wealth = -999999 if net_wealth==.
keep id year hwgt hh_lincome hearn_inc means hotherw_inc hssi_inc hafdc_inc hfood_inc hwic_inc hunemp_inc age educ emp hnp hpov htot_inc net_wealth hh_means di
order id year hwgt hh_lincome hearn_inc means hotherw_inc hssi_inc hafdc_inc hfood_inc hwic_inc hunemp_inc age educ emp hnp hpov htot_inc net_wealth hh_means di
outsheet using w_mt_dist, comma nolabel nonames replace
restore

*retirement income
preserve
sort id wave
keep id wave age hearn_inc hpension_p_inc hpension_g_inc hsose_inc hwgt educ emp hret_inc
order id wave age hearn_inc hpension_p_inc hpension_g_inc hsose_inc hret_inc hwgt educ emp
outsheet using ret_het, comma nolabel nonames replace
restore

*share of MT in income
preserve
gen means = hssi_inc+hotherw_inc+hafdc_inc+hfood_inc+hwic_inc
keep if means>0
replace hh_lincome=hh_lincome+means
gen share_means = means/hh_lincome
sum share_means
restore

*wealth of old, OLF
preserve
drop if net_wealth==. 
keep if age>49 & age<65
keep if lout==1
sum net_wealth hh_theq, detail
restore

*wealth of DI
preserve
drop if net_wealth==. 
keep if age>49 & age<65
keep if di==1
sum net_wealth hh_theq, detail
restore

*financial/durable for AMT households
preserve
gen means = hssi_inc+hotherw_inc+hafdc_inc+hfood_inc+hwic_inc
keep if means>0
drop if net_wealth==. 
gen share = net_wealth/(hh_theq+net_wealth)
sum share, detail
restore

*financial/durable for low income households
preserve
egen perc25 = pctile(hh_lincome), p(25)
keep if hh_lincome<=perc25
drop if net_wealth==. 
gen share = net_wealth/(hh_theq+net_wealth)
sum share, detail
restore
